@@ -20,7 +20,7 @@ package org.apache.zeppelin.flink;
 
 import org.apache.commons.cli.CommandLine;
 import org.apache.commons.compress.utils.Lists;
-import org.apache.flink.api.common.ExecutionConfig;
+import org.apache.commons.lang3.StringUtils;
 import org.apache.flink.api.common.JobStatus;
 import org.apache.flink.api.common.typeutils.TypeSerializer;
 import org.apache.flink.api.java.tuple.Tuple2;
@@ -50,6 +50,7 @@ import org.apache.flink.table.catalog.CatalogManager;
 import org.apache.flink.table.catalog.GenericInMemoryCatalog;
 import org.apache.flink.table.delegation.Parser;
 import org.apache.flink.table.functions.AggregateFunction;
+import org.apache.flink.table.functions.ScalarFunction;
 import org.apache.flink.table.functions.TableAggregateFunction;
 import org.apache.flink.table.functions.TableFunction;
 import org.apache.flink.table.operations.CatalogSinkModifyOperation;
@@ -79,10 +80,11 @@ import org.apache.flink.table.operations.ddl.DropTableOperation;
 import org.apache.flink.table.operations.ddl.DropTempSystemFunctionOperation;
 import org.apache.flink.table.operations.ddl.DropViewOperation;
 import org.apache.flink.table.sinks.TableSink;
+import org.apache.flink.table.utils.PrintUtils;
 import org.apache.flink.types.Row;
 import org.apache.flink.types.RowKind;
 import org.apache.flink.util.FlinkException;
-import org.apache.zeppelin.flink.shims111.CollectStreamTableSink;
+import org.apache.zeppelin.flink.shims112.CollectStreamTableSink;
 import org.apache.zeppelin.flink.shims112.Flink112ScalaShims;
 import org.apache.zeppelin.flink.sql.SqlCommandParser;
 import org.apache.zeppelin.flink.sql.SqlCommandParser.SqlCommand;
@@ -172,21 +174,32 @@ public class Flink112Shims extends FlinkShims {
 
   @Override
   public String getPyFlinkPythonPath(Properties properties) throws IOException {
+    String mode = properties.getProperty("flink.execution.mode");
+    if ("yarn-application".equalsIgnoreCase(mode)) {
+      // for yarn application mode, FLINK_HOME is container working directory
+      String flinkHome = new File(".").getAbsolutePath();
+      return getPyFlinkPythonPath(flinkHome + "/lib/python");
+    }
+
     String flinkHome = System.getenv("FLINK_HOME");
-    if (flinkHome != null) {
-      List<File> depFiles = null;
-      depFiles = Arrays.asList(new File(flinkHome + "/opt/python").listFiles());
-      StringBuilder builder = new StringBuilder();
-      for (File file : depFiles) {
-        LOGGER.info("Adding extracted file to PYTHONPATH: " + file.getAbsolutePath());
-        builder.append(file.getAbsolutePath() + ":");
-      }
-      return builder.toString();
+    if (StringUtils.isNotBlank(flinkHome)) {
+      return getPyFlinkPythonPath(flinkHome + "/opt/python");
     } else {
       throw new IOException("No FLINK_HOME is specified");
     }
   }
 
+  private String getPyFlinkPythonPath(String pyFlinkFolder) {
+    LOGGER.info("Getting pyflink lib from {}", pyFlinkFolder);
+    List<File> depFiles = Arrays.asList(new File(pyFlinkFolder).listFiles());
+    StringBuilder builder = new StringBuilder();
+    for (File file : depFiles) {
+      LOGGER.info("Adding extracted file {} to PYTHONPATH", file.getAbsolutePath());
+      builder.append(file.getAbsolutePath() + ":");
+    }
+    return builder.toString();
+  }
+
   @Override
   public Object getCollectStreamTableSink(InetAddress targetAddress, int targetPort, Object serializer) {
     return new CollectStreamTableSink(targetAddress, targetPort, (TypeSerializer<Tuple2<Boolean, Row>>) serializer);
@@ -247,6 +260,11 @@ public class Flink112Shims extends FlinkShims {
             .registerTableSinkInternal(tableName, (TableSink) collectTableSink);
   }
 
+  @Override
+  public void registerScalarFunction(Object btenv, String name, Object scalarFunction) {
+    ((StreamTableEnvironmentImpl)(btenv)).createTemporarySystemFunction(name, (ScalarFunction) scalarFunction);
+  }
+
   @Override
   public void registerTableFunction(Object btenv, String name, Object tableFunction) {
     ((StreamTableEnvironmentImpl) (btenv)).registerFunction(name, (TableFunction) tableFunction);
@@ -439,7 +457,11 @@ public class Flink112Shims extends FlinkShims {
     Map<String, ConfigOption> configOptions = new HashMap<>();
     configOptions.putAll(extractConfigOptions(ExecutionConfigOptions.class));
     configOptions.putAll(extractConfigOptions(OptimizerConfigOptions.class));
-    configOptions.putAll(extractConfigOptions(PythonOptions.class));
+    try {
+      configOptions.putAll(extractConfigOptions(PythonOptions.class));
+    } catch (NoClassDefFoundError e) {
+      LOGGER.warn("No pyflink jars found");
+    }
     configOptions.putAll(extractConfigOptions(TableConfigOptions.class));
     return configOptions;
   }
@@ -459,4 +481,9 @@ public class Flink112Shims extends FlinkShims {
     }
     return configOptions;
   }
+
+  @Override
+  public String[] rowToString(Object row, Object table, Object tableConfig) {
+    return PrintUtils.rowToString((Row) row);
+  }
 }
