@@ -182,17 +182,25 @@ public class JDBCInterpreter extends KerberosInterpreter {
 
   @Override
   protected boolean runKerberosLogin() {
+    // Initialize UGI before using
+    Configuration conf = new org.apache.hadoop.conf.Configuration();
+    conf.set("hadoop.security.authentication", KERBEROS.toString());
+    UserGroupInformation.setConfiguration(conf);
     try {
       if (UserGroupInformation.isLoginKeytabBased()) {
+        LOGGER.debug("Trying relogin from keytab");
         UserGroupInformation.getLoginUser().reloginFromKeytab();
         return true;
       } else if (UserGroupInformation.isLoginTicketBased()) {
+        LOGGER.debug("Trying relogin from ticket cache");
         UserGroupInformation.getLoginUser().reloginFromTicketCache();
         return true;
       }
     } catch (Exception e) {
       LOGGER.error("Unable to run kinit for zeppelin", e);
     }
+    LOGGER.debug("Neither Keytab nor ticket based login. " +
+        "runKerberosLogin() returning false");
     return false;
   }
 
@@ -342,6 +350,23 @@ public class JDBCInterpreter extends KerberosInterpreter {
     }
   }
 
+  /* Get user of this sql.
+   * 1. If shiro is enabled, use the login user
+   * 2. Otherwise try to get it from interpreter setting, e.g. default.user
+   */
+  private String getUser(InterpreterContext context) {
+    String user = context.getAuthenticationInfo().getUser();
+    String dbPrefix = getDBPrefix(context);
+
+    if ("anonymous".equalsIgnoreCase(user) && basePropertiesMap.containsKey(dbPrefix)) {
+      String userInProperty = basePropertiesMap.get(dbPrefix).getProperty(USER_KEY);
+      if (StringUtils.isNotBlank(userInProperty)) {
+        user = userInProperty;
+      }
+    }
+    return user;
+  }
+
   private String getEntityName(String replName, String propertyKey) {
     if ("jdbc".equals(replName)) {
       return propertyKey;
@@ -394,8 +419,7 @@ public class JDBCInterpreter extends KerberosInterpreter {
   private void setUserProperty(String dbPrefix, InterpreterContext context)
       throws SQLException, IOException, InterpreterException {
 
-    String user = context.getAuthenticationInfo().getUser();
-
+    String user = getUser(context);
     JDBCUserConfigurations jdbcUserConfigurations = getJDBCConfiguration(user);
     if (basePropertiesMap.get(dbPrefix).containsKey(USER_KEY) &&
         !basePropertiesMap.get(dbPrefix).getProperty(USER_KEY).isEmpty()) {
@@ -408,11 +432,11 @@ public class JDBCInterpreter extends KerberosInterpreter {
     if (existAccountInBaseProperty(dbPrefix)) {
       return;
     }
-    jdbcUserConfigurations.cleanUserProperty(dbPrefix);
 
     UsernamePassword usernamePassword = getUsernamePassword(context,
             getEntityName(context.getReplName(), dbPrefix));
     if (usernamePassword != null) {
+      jdbcUserConfigurations.cleanUserProperty(dbPrefix);
       jdbcUserConfigurations.setUserProperty(dbPrefix, usernamePassword);
     } else {
       closeDBPool(user, dbPrefix);
@@ -492,28 +516,34 @@ public class JDBCInterpreter extends KerberosInterpreter {
 
   public Connection getConnection(String dbPrefix, InterpreterContext context)
       throws ClassNotFoundException, SQLException, InterpreterException, IOException {
-    final String user =  context.getAuthenticationInfo().getUser();
-    Connection connection = null;
+
     if (dbPrefix == null || basePropertiesMap.get(dbPrefix) == null) {
+      LOGGER.warn("No such dbPrefix: {}", dbPrefix);
       return null;
     }
 
+    Connection connection = null;
+    String user = getUser(context);
     JDBCUserConfigurations jdbcUserConfigurations = getJDBCConfiguration(user);
     setUserProperty(dbPrefix, context);
 
     final Properties properties = jdbcUserConfigurations.getPropertyMap(dbPrefix);
     String url = properties.getProperty(URL_KEY);
-    String connectionUrl = appendProxyUserToURL(url, user, dbPrefix);
+    url = appendProxyUserToURL(url, user, dbPrefix);
+    String connectionUrl = appendTagsToURL(url, context);
 
-    String authType = properties.getProperty("zeppelin.jdbc.auth.type", "SIMPLE")
+    String authType = getProperty("zeppelin.jdbc.auth.type", "SIMPLE")
             .trim().toUpperCase();
     switch (authType) {
       case "SIMPLE":
         connection = getConnectionFromPool(connectionUrl, user, dbPrefix, properties);
         break;
       case "KERBEROS":
+        LOGGER.debug("Calling createSecureConfiguration(); this will do " +
+            "loginUserFromKeytab() if required");
         JDBCSecurityImpl.createSecureConfiguration(getProperties(),
                 UserGroupInformation.AuthenticationMethod.KERBEROS);
+        LOGGER.debug("createSecureConfiguration() returned");
         boolean isProxyEnabled = Boolean.parseBoolean(
                 getProperty("zeppelin.jdbc.auth.kerberos.proxy.enable", "true"));
         if (basePropertiesMap.get(dbPrefix).containsKey("proxy.user.property")
@@ -530,9 +560,10 @@ public class JDBCInterpreter extends KerberosInterpreter {
           }
 
           final String poolKey = dbPrefix;
+          final String finalUser = user;
           try {
             connection = ugi.doAs((PrivilegedExceptionAction<Connection>) () ->
-                    getConnectionFromPool(connectionUrl, user, poolKey, properties));
+                    getConnectionFromPool(connectionUrl, finalUser, poolKey, properties));
           } catch (Exception e) {
             LOGGER.error("Error in doAs", e);
             throw new InterpreterException("Error in doAs", e);
@@ -567,6 +598,24 @@ public class JDBCInterpreter extends KerberosInterpreter {
     return connectionUrl.toString();
   }
 
+  // only add tags for hive jdbc
+  private String appendTagsToURL(String url, InterpreterContext context) {
+    StringBuilder builder = new StringBuilder(url);
+    if (url.startsWith("jdbc:hive2:")) {
+      Integer lastIndexOfQMark = builder.indexOf("?");
+      if (lastIndexOfQMark == -1) {
+        builder.append("?");
+        lastIndexOfQMark = builder.length();
+      } else {
+        lastIndexOfQMark++;
+      }
+      builder.insert(lastIndexOfQMark, "mapreduce.job.tags=" + context.getParagraphId() + ";");
+      builder.insert(lastIndexOfQMark, "tez.application.tags=" + context.getParagraphId() + ";");
+    }
+    return builder.toString();
+  }
+
+
   private String getPassword(Properties properties) throws IOException, InterpreterException {
     if (isNotEmpty(properties.getProperty(PASSWORD_KEY))) {
       return properties.getProperty(PASSWORD_KEY);
@@ -692,7 +741,7 @@ public class JDBCInterpreter extends KerberosInterpreter {
     Statement statement;
     ResultSet resultSet = null;
     String paragraphId = context.getParagraphId();
-    String user = context.getAuthenticationInfo().getUser();
+    String user = getUser(context);
 
     try {
       connection = getConnection(dbPrefix, context);
@@ -716,6 +765,15 @@ public class JDBCInterpreter extends KerberosInterpreter {
     try {
       List<String>  sqlArray = sqlSplitter.splitSql(sql);
       for (String sqlToExecute : sqlArray) {
+        String sqlTrimmedLowerCase = sqlToExecute.trim().toLowerCase();
+        if (sqlTrimmedLowerCase.startsWith("set ") ||
+                sqlTrimmedLowerCase.startsWith("list ") ||
+                sqlTrimmedLowerCase.startsWith("add ") ||
+                sqlTrimmedLowerCase.startsWith("delete ")) {
+          // some version of hive doesn't work with set statement with empty line ahead.
+          // so we need to trim it first in this case.
+          sqlToExecute = sqlToExecute.trim();
+        }
         LOGGER.info("Execute sql: " + sqlToExecute);
         statement = connection.createStatement();
 
@@ -930,8 +988,7 @@ public class JDBCInterpreter extends KerberosInterpreter {
 
     LOGGER.info("Cancel current query statement.");
     String paragraphId = context.getParagraphId();
-    JDBCUserConfigurations jdbcUserConfigurations =
-            getJDBCConfiguration(context.getAuthenticationInfo().getUser());
+    JDBCUserConfigurations jdbcUserConfigurations = getJDBCConfiguration(getUser(context));
     try {
       jdbcUserConfigurations.cancelStatement(paragraphId);
     } catch (SQLException e) {
@@ -994,17 +1051,17 @@ public class JDBCInterpreter extends KerberosInterpreter {
 
   @Override
   public List<InterpreterCompletion> completion(String buf, int cursor,
-      InterpreterContext interpreterContext) throws InterpreterException {
+      InterpreterContext context) throws InterpreterException {
     List<InterpreterCompletion> candidates = new ArrayList<>();
-    String propertyKey = getDBPrefix(interpreterContext);
+    String propertyKey = getDBPrefix(context);
     String sqlCompleterKey =
-        String.format("%s.%s", interpreterContext.getAuthenticationInfo().getUser(), propertyKey);
+        String.format("%s.%s", getUser(context), propertyKey);
     SqlCompleter sqlCompleter = sqlCompletersMap.get(sqlCompleterKey);
 
     Connection connection = null;
     try {
-      if (interpreterContext != null) {
-        connection = getConnection(propertyKey, interpreterContext);
+      if (context != null) {
+        connection = getConnection(propertyKey, context);
       }
     } catch (ClassNotFoundException | SQLException | IOException e) {
       LOGGER.warn("SQLCompleter will created without use connection");
