@@ -17,7 +17,6 @@
 
 package org.apache.zeppelin.livy;
 
-
 import org.apache.commons.io.IOUtils;
 import org.apache.livy.test.framework.Cluster;
 import org.apache.livy.test.framework.Cluster$;
@@ -28,7 +27,6 @@ import org.apache.zeppelin.interpreter.InterpreterGroup;
 import org.apache.zeppelin.interpreter.InterpreterOutput;
 import org.apache.zeppelin.interpreter.InterpreterOutputListener;
 import org.apache.zeppelin.interpreter.InterpreterResult;
-import org.apache.zeppelin.interpreter.InterpreterResultMessage;
 import org.apache.zeppelin.interpreter.InterpreterResultMessageOutput;
 import org.apache.zeppelin.interpreter.LazyOpenInterpreter;
 import org.apache.zeppelin.interpreter.thrift.InterpreterCompletion;
@@ -49,8 +47,7 @@ import static org.junit.Assert.assertTrue;
 import static org.mockito.Mockito.mock;
 
 public class LivyInterpreterIT {
-
-  private static Logger LOGGER = LoggerFactory.getLogger(LivyInterpreterIT.class);
+  private static final Logger LOGGER = LoggerFactory.getLogger(LivyInterpreterIT.class);
   private static Cluster cluster;
   private static Properties properties;
 
@@ -102,8 +99,12 @@ public class LivyInterpreterIT {
     AuthenticationInfo authInfo = new AuthenticationInfo("user1");
     MyInterpreterOutputListener outputListener = new MyInterpreterOutputListener();
     InterpreterOutput output = new InterpreterOutput(outputListener);
-    InterpreterContext context = new InterpreterContext("noteId", "paragraphId", "livy.spark",
-        "title", "text", authInfo, null, null, null, null, null, null, output);
+    InterpreterContext context = InterpreterContext.builder()
+        .setNoteId("noteId")
+        .setParagraphId("paragraphId")
+        .setAuthenticationInfo(authInfo)
+        .setInterpreterOut(output)
+        .build();
     sparkInterpreter.open();
 
     LivySparkSQLInterpreter sqlInterpreter = new LivySparkSQLInterpreter(properties);
@@ -114,7 +115,7 @@ public class LivyInterpreterIT {
     try {
       // detect spark version
       InterpreterResult result = sparkInterpreter.interpret("sc.version", context);
-      assertEquals(InterpreterResult.Code.SUCCESS, result.code());
+      assertEquals(result.toString(), InterpreterResult.Code.SUCCESS, result.code());
       assertEquals(1, result.message().size());
 
       boolean isSpark2 = isSpark2(sparkInterpreter, context);
@@ -131,24 +132,29 @@ public class LivyInterpreterIT {
     AuthenticationInfo authInfo = new AuthenticationInfo("user1");
     MyInterpreterOutputListener outputListener = new MyInterpreterOutputListener();
     InterpreterOutput output = new InterpreterOutput(outputListener);
-    final InterpreterContext context = new InterpreterContext("noteId", "paragraphId", "livy.spark",
-        "title", "text", authInfo, null, null, null, null, null, null, output);
+    final InterpreterContext context = InterpreterContext.builder()
+        .setNoteId("noteId")
+        .setParagraphId("paragraphId")
+        .setAuthenticationInfo(authInfo)
+        .setInterpreterOut(output)
+        .build();
+    ;
 
     InterpreterResult result = sparkInterpreter.interpret("sc.parallelize(1 to 10).sum()", context);
-    assertEquals(InterpreterResult.Code.SUCCESS, result.code());
+    assertEquals(result.toString(), InterpreterResult.Code.SUCCESS, result.code());
     assertEquals(1, result.message().size());
     assertTrue(result.message().get(0).getData().contains("Double = 55.0"));
 
     // single line comment
     String singleLineComment = "println(1)// my comment";
     result = sparkInterpreter.interpret(singleLineComment, context);
-    assertEquals(InterpreterResult.Code.SUCCESS, result.code());
+    assertEquals(result.toString(), InterpreterResult.Code.SUCCESS, result.code());
     assertEquals(1, result.message().size());
 
     // multiple line comment
     String multipleLineComment = "println(1)/* multiple \n" + "line \n" + "comment */";
     result = sparkInterpreter.interpret(multipleLineComment, context);
-    assertEquals(InterpreterResult.Code.SUCCESS, result.code());
+    assertEquals(result.toString(), InterpreterResult.Code.SUCCESS, result.code());
     assertEquals(1, result.message().size());
 
     // multi-line string
@@ -156,7 +162,7 @@ public class LivyInterpreterIT {
         "line\"\"\"\n" +
         "println(str)";
     result = sparkInterpreter.interpret(multiLineString, context);
-    assertEquals(InterpreterResult.Code.SUCCESS, result.code());
+    assertEquals(result.toString(), InterpreterResult.Code.SUCCESS, result.code());
     assertEquals(1, result.message().size());
     assertTrue(result.message().get(0).getData().contains("multiple\nline"));
 
@@ -165,14 +171,14 @@ public class LivyInterpreterIT {
         "name:String)\n" +
         "val p=Person(1, \"name_a\")";
     result = sparkInterpreter.interpret(caseClassCode, context);
-    assertEquals(InterpreterResult.Code.SUCCESS, result.code());
+    assertEquals(result.toString(), InterpreterResult.Code.SUCCESS, result.code());
     assertEquals(1, result.message().size());
     assertTrue(result.message().get(0).getData().contains("p: Person = Person(1,name_a)"));
 
     // object class
     String objectClassCode = "object Person {}";
     result = sparkInterpreter.interpret(objectClassCode, context);
-    assertEquals(InterpreterResult.Code.SUCCESS, result.code());
+    assertEquals(result.toString(), InterpreterResult.Code.SUCCESS, result.code());
     assertEquals(1, result.message().size());
     if (!isSpark2) {
       assertTrue(result.message().get(0).getData().contains("defined module Person"));
@@ -183,7 +189,7 @@ public class LivyInterpreterIT {
     // html output
     String htmlCode = "println(\"%html <h1> hello </h1>\")";
     result = sparkInterpreter.interpret(htmlCode, context);
-    assertEquals(InterpreterResult.Code.SUCCESS, result.code());
+    assertEquals(result.toString(), InterpreterResult.Code.SUCCESS, result.code());
     assertEquals(1, result.message().size());
     assertEquals(InterpreterResult.Type.HTML, result.message().get(0).getType());
 
@@ -230,8 +236,12 @@ public class LivyInterpreterIT {
     AuthenticationInfo authInfo = new AuthenticationInfo("user1");
     MyInterpreterOutputListener outputListener = new MyInterpreterOutputListener();
     InterpreterOutput output = new InterpreterOutput(outputListener);
-    final InterpreterContext context = new InterpreterContext("noteId", "paragraphId", "livy.spark",
-        "title", "text", authInfo, null, null, null, null, null, null, output);
+    final InterpreterContext context = InterpreterContext.builder()
+        .setNoteId("noteId")
+        .setParagraphId("paragraphId")
+        .setAuthenticationInfo(authInfo)
+        .setInterpreterOut(output)
+        .build();
 
     InterpreterResult result = null;
     // test DataFrame api
@@ -239,7 +249,7 @@ public class LivyInterpreterIT {
       result = sparkInterpreter.interpret(
           "val df=sqlContext.createDataFrame(Seq((\"hello\",20))).toDF(\"col_1\", \"col_2\")\n"
               + "df.collect()", context);
-      assertEquals(InterpreterResult.Code.SUCCESS, result.code());
+      assertEquals(result.toString(), InterpreterResult.Code.SUCCESS, result.code());
       assertEquals(1, result.message().size());
       assertTrue(result.message().get(0).getData()
           .contains("Array[org.apache.spark.sql.Row] = Array([hello,20])"));
@@ -247,7 +257,7 @@ public class LivyInterpreterIT {
       result = sparkInterpreter.interpret(
           "val df=spark.createDataFrame(Seq((\"hello\",20))).toDF(\"col_1\", \"col_2\")\n"
               + "df.collect()", context);
-      assertEquals(InterpreterResult.Code.SUCCESS, result.code());
+      assertEquals(result.toString(), InterpreterResult.Code.SUCCESS, result.code());
       assertEquals(1, result.message().size());
       assertTrue(result.message().get(0).getData()
           .contains("Array[org.apache.spark.sql.Row] = Array([hello,20])"));
@@ -255,25 +265,25 @@ public class LivyInterpreterIT {
     sparkInterpreter.interpret("df.registerTempTable(\"df\")", context);
     // test LivySparkSQLInterpreter which share the same SparkContext with LivySparkInterpreter
     result = sqlInterpreter.interpret("select * from df where col_1='hello'", context);
-    assertEquals(InterpreterResult.Code.SUCCESS, result.code());
+    assertEquals(result.toString(), InterpreterResult.Code.SUCCESS, result.code());
     assertEquals(InterpreterResult.Type.TABLE, result.message().get(0).getType());
     assertEquals("col_1\tcol_2\nhello\t20", result.message().get(0).getData());
     // double quotes
     result = sqlInterpreter.interpret("select * from df where col_1=\"hello\"", context);
-    assertEquals(InterpreterResult.Code.SUCCESS, result.code());
+    assertEquals(result.toString(), InterpreterResult.Code.SUCCESS, result.code());
     assertEquals(InterpreterResult.Type.TABLE, result.message().get(0).getType());
     assertEquals("col_1\tcol_2\nhello\t20", result.message().get(0).getData());
 
     // only enable this test in spark2 as spark1 doesn't work for this case
     if (isSpark2) {
       result = sqlInterpreter.interpret("select * from df where col_1=\"he\\\"llo\" ", context);
-      assertEquals(InterpreterResult.Code.SUCCESS, result.code());
+      assertEquals(result.toString(), InterpreterResult.Code.SUCCESS, result.code());
       assertEquals(InterpreterResult.Type.TABLE, result.message().get(0).getType());
     }
 
     // single quotes inside attribute value
     result = sqlInterpreter.interpret("select * from df where col_1=\"he'llo\"", context);
-    assertEquals(InterpreterResult.Code.SUCCESS, result.code());
+    assertEquals(result.toString(), InterpreterResult.Code.SUCCESS, result.code());
     assertEquals(InterpreterResult.Type.TABLE, result.message().get(0).getType());
 
     // test sql with syntax error
@@ -315,25 +325,28 @@ public class LivyInterpreterIT {
     // test result string truncate
     if (!isSpark2) {
       result = sparkInterpreter.interpret(
-          "val df=sqlContext.createDataFrame(Seq((\"12characters12characters\",20))).toDF(\"col_1\", \"col_2\")\n"
+          "val df=sqlContext.createDataFrame(Seq((\"12characters12characters\",20)))"
+              + ".toDF(\"col_1\", \"col_2\")\n"
               + "df.collect()", context);
-      assertEquals(InterpreterResult.Code.SUCCESS, result.code());
+      assertEquals(result.toString(), InterpreterResult.Code.SUCCESS, result.code());
       assertEquals(1, result.message().size());
       assertTrue(result.message().get(0).getData()
           .contains("Array[org.apache.spark.sql.Row] = Array([12characters12characters,20])"));
     } else {
       result = sparkInterpreter.interpret(
-          "val df=spark.createDataFrame(Seq((\"12characters12characters\",20))).toDF(\"col_1\", \"col_2\")\n"
+          "val df=spark.createDataFrame(Seq((\"12characters12characters\",20)))"
+              + ".toDF(\"col_1\", \"col_2\")\n"
               + "df.collect()", context);
-      assertEquals(InterpreterResult.Code.SUCCESS, result.code());
+      assertEquals(result.toString(), InterpreterResult.Code.SUCCESS, result.code());
       assertEquals(1, result.message().size());
       assertTrue(result.message().get(0).getData()
           .contains("Array[org.apache.spark.sql.Row] = Array([12characters12characters,20])"));
     }
     sparkInterpreter.interpret("df.registerTempTable(\"df\")", context);
     // test LivySparkSQLInterpreter which share the same SparkContext with LivySparkInterpreter
-    result = sqlInterpreter.interpret("select * from df where col_1='12characters12characters'", context);
-    assertEquals(InterpreterResult.Code.SUCCESS, result.code());
+    result = sqlInterpreter.interpret("select * from df where col_1='12characters12characters'",
+        context);
+    assertEquals(result.toString(), InterpreterResult.Code.SUCCESS, result.code());
     assertEquals(InterpreterResult.Type.TABLE, result.message().get(0).getType());
     assertEquals("col_1\tcol_2\n12characters12cha...\t20", result.message().get(0).getData());
 
@@ -350,22 +363,28 @@ public class LivyInterpreterIT {
     AuthenticationInfo authInfo = new AuthenticationInfo("user1");
     MyInterpreterOutputListener outputListener = new MyInterpreterOutputListener();
     InterpreterOutput output = new InterpreterOutput(outputListener);
-    final InterpreterContext context = new InterpreterContext("noteId", "paragraphId", "livy.pyspark",
-        "title", "text", authInfo, null, null, null, null, null, null, output);
+    final InterpreterContext context = InterpreterContext.builder()
+        .setNoteId("noteId")
+        .setParagraphId("paragraphId")
+        .setAuthenticationInfo(authInfo)
+        .setInterpreterOut(output)
+        .build();
     pysparkInterpreter.open();
 
     // test traceback msg
     try {
       pysparkInterpreter.getLivyVersion();
-      // for livy version >=0.3 , input some erroneous spark code, check the shown result is more than one line
-      InterpreterResult result = pysparkInterpreter.interpret("sc.parallelize(wrongSyntax(1, 2)).count()", context);
+      // for livy version >=0.3 , input some erroneous spark code, check the shown result is more
+      // than one line
+      InterpreterResult result = pysparkInterpreter.interpret(
+          "sc.parallelize(wrongSyntax(1, 2)).count()", context);
       assertEquals(InterpreterResult.Code.ERROR, result.code());
       assertTrue(result.message().get(0).getData().split("\n").length > 1);
       assertTrue(result.message().get(0).getData().contains("Traceback"));
     } catch (APINotFoundException e) {
       // only livy 0.2 can throw this exception since it doesn't have /version endpoint
-      // in livy 0.2, most error msg is encapsulated in evalue field, only print(a) in pyspark would return none-empty
-      // traceback
+      // in livy 0.2, most error msg is encapsulated in evalue field, only print(a) in pyspark would
+      // return none-empty traceback
       InterpreterResult result = pysparkInterpreter.interpret("print(a)", context);
       assertEquals(InterpreterResult.Code.ERROR, result.code());
       assertTrue(result.message().get(0).getData().split("\n").length > 1);
@@ -386,14 +405,14 @@ public class LivyInterpreterIT {
 
     try {
       InterpreterResult result = pysparkInterpreter.interpret("sc.version", context);
-      assertEquals(InterpreterResult.Code.SUCCESS, result.code());
+      assertEquals(result.toString(), InterpreterResult.Code.SUCCESS, result.code());
       assertEquals(1, result.message().size());
 
       boolean isSpark2 = isSpark2(pysparkInterpreter, context);
 
       // test RDD api
       result = pysparkInterpreter.interpret("sc.range(1, 10).sum()", context);
-      assertEquals(InterpreterResult.Code.SUCCESS, result.code());
+      assertEquals(result.toString(), InterpreterResult.Code.SUCCESS, result.code());
       assertEquals(1, result.message().size());
       assertTrue(result.message().get(0).getData().contains("45"));
 
@@ -403,7 +422,7 @@ public class LivyInterpreterIT {
             + "sqlContext = SQLContext(sc)", context);
         result = pysparkInterpreter.interpret("df=sqlContext.createDataFrame([(\"hello\",20)])\n"
             + "df.collect()", context);
-        assertEquals(InterpreterResult.Code.SUCCESS, result.code());
+        assertEquals(result.toString(), InterpreterResult.Code.SUCCESS, result.code());
         assertEquals(1, result.message().size());
         //python2 has u and python3 don't have u
         assertTrue(result.message().get(0).getData().contains("[Row(_1=u'hello', _2=20)]")
@@ -411,7 +430,7 @@ public class LivyInterpreterIT {
       } else {
         result = pysparkInterpreter.interpret("df=spark.createDataFrame([(\"hello\",20)])\n"
             + "df.collect()", context);
-        assertEquals(InterpreterResult.Code.SUCCESS, result.code());
+        assertEquals(result.toString(), InterpreterResult.Code.SUCCESS, result.code());
         assertEquals(1, result.message().size());
         //python2 has u and python3 don't have u
         assertTrue(result.message().get(0).getData().contains("[Row(_1=u'hello', _2=20)]")
@@ -422,7 +441,7 @@ public class LivyInterpreterIT {
       pysparkInterpreter.interpret("t = [{\"name\":\"userA\", \"role\":\"roleA\"},"
           + "{\"name\":\"userB\", \"role\":\"roleB\"}]", context);
       result = pysparkInterpreter.interpret("%table t", context);
-      assertEquals(InterpreterResult.Code.SUCCESS, result.code());
+      assertEquals(result.toString(), InterpreterResult.Code.SUCCESS, result.code());
       assertEquals(1, result.message().size());
       assertEquals(InterpreterResult.Type.TABLE, result.message().get(0).getType());
       assertTrue(result.message().get(0).getData().contains("userA"));
@@ -463,7 +482,8 @@ public class LivyInterpreterIT {
   }
 
   @Test
-  public void testSparkInterpreterWithDisplayAppInfo_StringWithoutTruncation() throws InterpreterException {
+  public void testSparkInterpreterWithDisplayAppInfo_StringWithoutTruncation()
+      throws InterpreterException {
     if (!checkPreCondition()) {
       return;
     }
@@ -480,8 +500,12 @@ public class LivyInterpreterIT {
     AuthenticationInfo authInfo = new AuthenticationInfo("user1");
     MyInterpreterOutputListener outputListener = new MyInterpreterOutputListener();
     InterpreterOutput output = new InterpreterOutput(outputListener);
-    InterpreterContext context = new InterpreterContext("noteId", "paragraphId", "livy.spark",
-        "title", "text", authInfo, null, null, null, null, null, null, output);
+    InterpreterContext context = InterpreterContext.builder()
+        .setNoteId("noteId")
+        .setParagraphId("paragraphId")
+        .setAuthenticationInfo(authInfo)
+        .setInterpreterOut(output)
+        .build();
     sparkInterpreter.open();
 
     LivySparkSQLInterpreter sqlInterpreter = new LivySparkSQLInterpreter(properties2);
@@ -491,7 +515,7 @@ public class LivyInterpreterIT {
 
     try {
       InterpreterResult result = sparkInterpreter.interpret("sc.version", context);
-      assertEquals(InterpreterResult.Code.SUCCESS, result.code());
+      assertEquals(result.toString(), InterpreterResult.Code.SUCCESS, result.code());
       assertEquals(2, result.message().size());
       // check yarn appId and ensure it is not null
       assertTrue(result.message().get(1).getData().contains("Spark Application Id: application_"));
@@ -499,38 +523,41 @@ public class LivyInterpreterIT {
       // html output
       String htmlCode = "println(\"%html <h1> hello </h1>\")";
       result = sparkInterpreter.interpret(htmlCode, context);
-      assertEquals(InterpreterResult.Code.SUCCESS, result.code());
+      assertEquals(result.toString(), InterpreterResult.Code.SUCCESS, result.code());
       assertEquals(2, result.message().size());
       assertEquals(InterpreterResult.Type.HTML, result.message().get(0).getType());
 
       // detect spark version
       result = sparkInterpreter.interpret("sc.version", context);
-      assertEquals(InterpreterResult.Code.SUCCESS, result.code());
+      assertEquals(result.toString(), InterpreterResult.Code.SUCCESS, result.code());
       assertEquals(2, result.message().size());
 
       boolean isSpark2 = isSpark2(sparkInterpreter, context);
 
       if (!isSpark2) {
         result = sparkInterpreter.interpret(
-            "val df=sqlContext.createDataFrame(Seq((\"12characters12characters\",20))).toDF(\"col_1\", \"col_2\")\n"
+            "val df=sqlContext.createDataFrame(Seq((\"12characters12characters\",20)))"
+                + ".toDF(\"col_1\", \"col_2\")\n"
                 + "df.collect()", context);
-        assertEquals(InterpreterResult.Code.SUCCESS, result.code());
+        assertEquals(result.toString(), InterpreterResult.Code.SUCCESS, result.code());
         assertEquals(2, result.message().size());
         assertTrue(result.message().get(0).getData()
             .contains("Array[org.apache.spark.sql.Row] = Array([12characters12characters,20])"));
       } else {
         result = sparkInterpreter.interpret(
-            "val df=spark.createDataFrame(Seq((\"12characters12characters\",20))).toDF(\"col_1\", \"col_2\")\n"
+            "val df=spark.createDataFrame(Seq((\"12characters12characters\",20)))"
+                + ".toDF(\"col_1\", \"col_2\")\n"
                 + "df.collect()", context);
-        assertEquals(InterpreterResult.Code.SUCCESS, result.code());
+        assertEquals(result.toString(), InterpreterResult.Code.SUCCESS, result.code());
         assertEquals(2, result.message().size());
         assertTrue(result.message().get(0).getData()
             .contains("Array[org.apache.spark.sql.Row] = Array([12characters12characters,20])"));
       }
       sparkInterpreter.interpret("df.registerTempTable(\"df\")", context);
       // test LivySparkSQLInterpreter which share the same SparkContext with LivySparkInterpreter
-      result = sqlInterpreter.interpret("select * from df where col_1='12characters12characters'", context);
-      assertEquals(InterpreterResult.Code.SUCCESS, result.code());
+      result = sqlInterpreter.interpret("select * from df where col_1='12characters12characters'",
+          context);
+      assertEquals(result.toString(), InterpreterResult.Code.SUCCESS, result.code());
       assertEquals(InterpreterResult.Type.TABLE, result.message().get(0).getType());
       assertEquals("col_1\tcol_2\n12characters12characters\t20", result.message().get(0).getData());
     } finally {
@@ -557,8 +584,12 @@ public class LivyInterpreterIT {
     AuthenticationInfo authInfo = new AuthenticationInfo("user1");
     MyInterpreterOutputListener outputListener = new MyInterpreterOutputListener();
     InterpreterOutput output = new InterpreterOutput(outputListener);
-    final InterpreterContext context = new InterpreterContext("noteId", "paragraphId", "livy.sparkr",
-        "title", "text", authInfo, null, null, null, null, null, null, output);
+    final InterpreterContext context = InterpreterContext.builder()
+        .setNoteId("noteId")
+        .setParagraphId("paragraphId")
+        .setAuthenticationInfo(authInfo)
+        .setInterpreterOut(output)
+        .build();
     sparkRInterpreter.open();
 
     try {
@@ -568,7 +599,7 @@ public class LivyInterpreterIT {
       // test DataFrame api
       if (isSpark2) {
         result = sparkRInterpreter.interpret("df <- as.DataFrame(faithful)\nhead(df)", context);
-        assertEquals(InterpreterResult.Code.SUCCESS, result.code());
+        assertEquals(result.toString(), InterpreterResult.Code.SUCCESS, result.code());
         assertEquals(1, result.message().size());
         assertTrue(result.message().get(0).getData().contains("eruptions waiting"));
 
@@ -597,7 +628,7 @@ public class LivyInterpreterIT {
       } else {
         result = sparkRInterpreter.interpret("df <- createDataFrame(sqlContext, faithful)" +
             "\nhead(df)", context);
-        assertEquals(InterpreterResult.Code.SUCCESS, result.code());
+        assertEquals(result.toString(), InterpreterResult.Code.SUCCESS, result.code());
         assertEquals(1, result.message().size());
         assertTrue(result.message().get(0).getData().contains("eruptions waiting"));
       }
@@ -606,8 +637,7 @@ public class LivyInterpreterIT {
       result = sparkRInterpreter.interpret("cat(a)", context);
       assertEquals(InterpreterResult.Code.ERROR, result.code());
       assertEquals(InterpreterResult.Type.TEXT, result.message().get(0).getType());
-      assertTrue("Actual Result: " + result.message().get(0).getData(),
-              result.message().get(0).getData().contains("object 'a' not found"));
+      assertTrue(result.message().get(0).getData().contains("object 'a' not found"));
     } finally {
       sparkRInterpreter.close();
     }
@@ -634,16 +664,20 @@ public class LivyInterpreterIT {
       AuthenticationInfo authInfo = new AuthenticationInfo("user1");
       MyInterpreterOutputListener outputListener = new MyInterpreterOutputListener();
       InterpreterOutput output = new InterpreterOutput(outputListener);
-      InterpreterContext context = new InterpreterContext("noteId", "paragraphId", "livy.sql",
-          "title", "text", authInfo, null, null, null, null, null, null, output);
+      InterpreterContext context = InterpreterContext.builder()
+          .setNoteId("noteId")
+          .setParagraphId("paragraphId")
+          .setAuthenticationInfo(authInfo)
+          .setInterpreterOut(output)
+          .build();
 
       String p1 = IOUtils.toString(getClass().getResourceAsStream("/livy_tutorial_1.scala"));
       InterpreterResult result = sparkInterpreter.interpret(p1, context);
-      assertEquals(InterpreterResult.Code.SUCCESS, result.code());
+      assertEquals(result.toString(), InterpreterResult.Code.SUCCESS, result.code());
 
       String p2 = IOUtils.toString(getClass().getResourceAsStream("/livy_tutorial_2.sql"));
       result = sqlInterpreter.interpret(p2, context);
-      assertEquals(InterpreterResult.Code.SUCCESS, result.code());
+      assertEquals(result.toString(), InterpreterResult.Code.SUCCESS, result.code());
       assertEquals(InterpreterResult.Type.TABLE, result.message().get(0).getType());
     } finally {
       sparkInterpreter.close();
@@ -692,28 +726,34 @@ public class LivyInterpreterIT {
       AuthenticationInfo authInfo = new AuthenticationInfo("user1");
       MyInterpreterOutputListener outputListener = new MyInterpreterOutputListener();
       InterpreterOutput output = new InterpreterOutput(outputListener);
-      InterpreterContext context = new InterpreterContext("noteId", "paragraphId", "livy.sql",
-          "title", "text", authInfo, null, null, null, null, null, null, output);
+      InterpreterContext context = InterpreterContext.builder()
+          .setNoteId("noteId")
+          .setParagraphId("paragraphId")
+          .setAuthenticationInfo(authInfo)
+          .setInterpreterOut(output)
+          .build();
       // detect spark version
       InterpreterResult result = sparkInterpreter.interpret("sc.version", context);
-      assertEquals(InterpreterResult.Code.SUCCESS, result.code());
+      assertEquals(result.toString(), InterpreterResult.Code.SUCCESS, result.code());
       assertEquals(1, result.message().size());
 
-      boolean isSpark2 = isSpark2((BaseLivyInterpreter) sparkInterpreter.getInnerInterpreter(), context);
+      boolean isSpark2 = isSpark2((BaseLivyInterpreter) sparkInterpreter.getInnerInterpreter(),
+          context);
 
       if (!isSpark2) {
         result = sparkInterpreter.interpret(
             "val df=sqlContext.createDataFrame(Seq((\"hello\",20))).toDF(\"col_1\", \"col_2\")\n"
                 + "df.collect()", context);
-        assertEquals(InterpreterResult.Code.SUCCESS, result.code());
+        assertEquals(result.toString(), InterpreterResult.Code.SUCCESS, result.code());
         assertEquals(1, result.message().size());
         assertTrue(result.message().get(0).getData()
             .contains("Array[org.apache.spark.sql.Row] = Array([hello,20])"));
         sparkInterpreter.interpret("df.registerTempTable(\"df\")", context);
 
         // access table from pyspark
-        result = pysparkInterpreter.interpret("sqlContext.sql(\"select * from df\").show()", context);
-        assertEquals(InterpreterResult.Code.SUCCESS, result.code());
+        result = pysparkInterpreter.interpret("sqlContext.sql(\"select * from df\").show()",
+            context);
+        assertEquals(result.toString(), InterpreterResult.Code.SUCCESS, result.code());
         assertEquals(1, result.message().size());
         assertTrue(result.message().get(0).getData()
             .contains("+-----+-----+\n" +
@@ -723,15 +763,16 @@ public class LivyInterpreterIT {
                 "+-----+-----+"));
 
         // access table from sparkr
-        result = sparkRInterpreter.interpret("head(sql(sqlContext, \"select * from df\"))", context);
-        assertEquals(InterpreterResult.Code.SUCCESS, result.code());
+        result = sparkRInterpreter.interpret("head(sql(sqlContext, \"select * from df\"))",
+            context);
+        assertEquals(result.toString(), InterpreterResult.Code.SUCCESS, result.code());
         assertEquals(1, result.message().size());
         assertTrue(result.message().get(0).getData().contains("col_1 col_2\n1 hello    20"));
       } else {
         result = sparkInterpreter.interpret(
             "val df=spark.createDataFrame(Seq((\"hello\",20))).toDF(\"col_1\", \"col_2\")\n"
                 + "df.collect()", context);
-        assertEquals(InterpreterResult.Code.SUCCESS, result.code());
+        assertEquals(result.toString(), InterpreterResult.Code.SUCCESS, result.code());
         assertEquals(1, result.message().size());
         assertTrue(result.message().get(0).getData()
             .contains("Array[org.apache.spark.sql.Row] = Array([hello,20])"));
@@ -739,7 +780,7 @@ public class LivyInterpreterIT {
 
         // access table from pyspark
         result = pysparkInterpreter.interpret("spark.sql(\"select * from df\").show()", context);
-        assertEquals(InterpreterResult.Code.SUCCESS, result.code());
+        assertEquals(result.toString(), InterpreterResult.Code.SUCCESS, result.code());
         assertEquals(1, result.message().size());
         assertTrue(result.message().get(0).getData()
             .contains("+-----+-----+\n" +
@@ -750,7 +791,7 @@ public class LivyInterpreterIT {
 
         // access table from sparkr
         result = sparkRInterpreter.interpret("head(sql(\"select * from df\"))", context);
-        assertEquals(InterpreterResult.Code.SUCCESS, result.code());
+        assertEquals(result.toString(), InterpreterResult.Code.SUCCESS, result.code());
         assertEquals(1, result.message().size());
         assertTrue(result.message().get(0).getData().contains("col_1 col_2\n1 hello    20"));
       }
@@ -758,19 +799,19 @@ public class LivyInterpreterIT {
       // test plotting of python
       result = pysparkInterpreter.interpret(
           "import matplotlib.pyplot as plt\n" +
-          "plt.switch_backend('agg')\n" +
-          "data=[1,2,3,4]\n" +
-          "plt.figure()\n" +
-          "plt.plot(data)\n" +
-          "%matplot plt", context);
-      assertEquals(InterpreterResult.Code.SUCCESS, result.code());
+              "plt.switch_backend('agg')\n" +
+              "data=[1,2,3,4]\n" +
+              "plt.figure()\n" +
+              "plt.plot(data)\n" +
+              "%matplot plt", context);
+      assertEquals(result.toString(), InterpreterResult.Code.SUCCESS, result.code());
       assertEquals(1, result.message().size());
       assertEquals(InterpreterResult.Type.IMG, result.message().get(0).getType());
 
       // test plotting of R
       result = sparkRInterpreter.interpret(
           "hist(mtcars$mpg)", context);
-      assertEquals(InterpreterResult.Code.SUCCESS, result.code());
+      assertEquals(result.toString(), InterpreterResult.Code.SUCCESS, result.code());
       assertEquals(1, result.message().size());
       assertEquals(InterpreterResult.Type.IMG, result.message().get(0).getType());
 
@@ -787,22 +828,13 @@ public class LivyInterpreterIT {
   }
 
   private boolean isSpark2(BaseLivyInterpreter interpreter, InterpreterContext context) {
-    InterpreterResult result = null;
     if (interpreter instanceof LivySparkRInterpreter) {
-      result = interpreter.interpret("sparkR.session()", context);
+      InterpreterResult result = interpreter.interpret("sparkR.session()", context);
       // SparkRInterpreter would always return SUCCESS, it is due to bug of LIVY-313
-      if (result.message().get(0).getData().contains("Error")) {
-        return false;
-      } else {
-        return true;
-      }
+      return !result.message().get(0).getData().contains("Error");
     } else {
-      result = interpreter.interpret("spark", context);
-      if (result.code() == InterpreterResult.Code.SUCCESS) {
-        return true;
-      } else {
-        return false;
-      }
+      InterpreterResult result = interpreter.interpret("spark", context);
+      return result.code() == InterpreterResult.Code.SUCCESS;
     }
   }
 
