@@ -80,16 +80,24 @@ public class PythonInterpreter extends Interpreter {
   public void open() throws InterpreterException {
     // try IPythonInterpreter first
     iPythonInterpreter = getIPythonInterpreter();
-    if (getProperty("zeppelin.python.useIPython", "true").equals("true") &&
-        StringUtils.isEmpty(
-            iPythonInterpreter.checkKernelPrerequisite(getPythonExec()))) {
-      try {
-        iPythonInterpreter.open();
-        LOGGER.info("IPython is available, Use IPythonInterpreter to replace PythonInterpreter");
-        return;
-      } catch (Exception e) {
-        iPythonInterpreter = null;
-        LOGGER.warn("Fail to open IPythonInterpreter", e);
+    boolean useIPython = Boolean.parseBoolean(getProperty("zeppelin.python.useIPython", "true"));
+
+    LOGGER.info("zeppelin.python.useIPython: {}", useIPython);
+    if (useIPython) {
+      String checkKernelPrerequisiteResult = iPythonInterpreter.checkKernelPrerequisite(
+              getPythonExec());
+      if (StringUtils.isEmpty(checkKernelPrerequisiteResult)) {
+        try {
+          iPythonInterpreter.open();
+          LOGGER.info("IPython is available, Use IPythonInterpreter to replace PythonInterpreter");
+          return;
+        } catch (Exception e) {
+          iPythonInterpreter = null;
+          LOGGER.warn("Fail to open IPythonInterpreter", e);
+        }
+      } else {
+        LOGGER.info("IPython requirement is not met, checkKernelPrerequisiteResult: {}",
+                checkKernelPrerequisiteResult);
       }
     }
 
@@ -223,9 +231,12 @@ public class PythonInterpreter extends Interpreter {
 
   // Run python script
   // Choose python in the order of
-  // condaPythonExec > zeppelin.python
+  // {conda.env.name}/bin/python > condaPythonExec > zeppelin.python
   protected String getPythonExec() {
-    if (condaPythonExec != null) {
+    String condaEnv = getProperty("zeppelin.interpreter.conda.env.name");
+    if (StringUtils.isNotBlank(condaEnv)) {
+      return condaEnv + "/bin/python";
+    } else if (condaPythonExec != null) {
       return condaPythonExec;
     } else {
       return getProperty("zeppelin.python", "python");
@@ -552,13 +563,23 @@ public class PythonInterpreter extends Interpreter {
     String bootstrapCode =
         IOUtils.toString(getClass().getClassLoader().getResourceAsStream(resourceName));
     try {
-      // Add hook explicitly, otherwise python will fail to execute the statement
-      InterpreterResult result = interpret(bootstrapCode + "\n" + "__zeppelin__._displayhook()",
-          InterpreterContext.get());
-      if (result.code() != Code.SUCCESS) {
-        throw new IOException("Fail to run bootstrap script: " + resourceName + "\n" + result);
+      if (iPythonInterpreter != null) {
+        InterpreterResult result = iPythonInterpreter.interpret(bootstrapCode,
+                InterpreterContext.get());
+        if (result.code() != Code.SUCCESS) {
+          throw new IOException("Fail to run bootstrap script: " + resourceName + "\n" + result);
+        } else {
+          LOGGER.debug("Bootstrap python successfully.");
+        }
       } else {
-        LOGGER.debug("Bootstrap python successfully.");
+        // Add hook explicitly, otherwise python will fail to execute the statement
+        InterpreterResult result = interpret(bootstrapCode + "\n" + "__zeppelin__._displayhook()",
+                InterpreterContext.get());
+        if (result.code() != Code.SUCCESS) {
+          throw new IOException("Fail to run bootstrap script: " + resourceName + "\n" + result);
+        } else {
+          LOGGER.debug("Bootstrap python successfully.");
+        }
       }
     } catch (InterpreterException e) {
       throw new IOException(e);
