@@ -18,7 +18,7 @@
 package org.apache.zeppelin.sap.universe;
 
 import jline.console.completer.ArgumentCompleter.WhitespaceArgumentDelimiter;
-import org.apache.commons.lang.StringUtils;
+import org.apache.commons.lang3.StringUtils;
 
 import java.util.*;
 
@@ -232,8 +232,7 @@ public class UniverseUtil {
           continue;
         }
         if (pathClosed && singleQuoteClosed && c == ',') {
-          select.append(parseResultObj(resultObj.toString().replaceAll("(?i)distinc", ""),
-            nodeInfos));
+          select.append(parseResultObj(resultObj.toString().replaceAll("(?i)distinc", ""), nodeInfos));
           resultObj = new StringBuilder();
         } else {
           resultObj.append(c);
