@@ -17,13 +17,22 @@
 
 package org.apache.zeppelin.display;
 
-import org.apache.commons.lang.StringUtils;
-import org.apache.zeppelin.common.JsonSerializable;
-import org.apache.zeppelin.display.ui.*;
+import org.apache.commons.lang3.StringUtils;
+import org.apache.zeppelin.display.ui.CheckBox;
+import org.apache.zeppelin.display.ui.OptionInput;
 import org.apache.zeppelin.display.ui.OptionInput.ParamOption;
+import org.apache.zeppelin.display.ui.Password;
+import org.apache.zeppelin.display.ui.Select;
+import org.apache.zeppelin.display.ui.TextBox;
 
 import java.io.Serializable;
-import java.util.*;
+import java.util.ArrayList;
+import java.util.Arrays;
+import java.util.Collection;
+import java.util.LinkedHashMap;
+import java.util.LinkedList;
+import java.util.List;
+import java.util.Map;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
 
@@ -41,6 +50,7 @@ public class Input<T> implements Serializable {
           .registerSubtype(TextBox.class, "TextBox")
           .registerSubtype(Select.class, "Select")
           .registerSubtype(CheckBox.class, "CheckBox")
+          .registerSubtype(Password.class, "Password")
           .registerSubtype(OldInput.OldTextBox.class, "input")
           .registerSubtype(OldInput.OldSelect.class, "select")
           .registerSubtype(OldInput.OldCheckBox.class, "checkbox")
@@ -210,7 +220,7 @@ public class Input<T> implements Serializable {
     String displayName = null;
     String type = null;
     String arg = null;
-    Object defaultValue = "";
+    Object defaultValue = null;
     ParamOption[] paramOptions = null;
 
     // get var name type
@@ -274,11 +284,17 @@ public class Input<T> implements Serializable {
       }
     } else if (type.equals("checkbox")) {
       input = new CheckBox(varName, (Object[]) defaultValue, paramOptions);
+    } else if (type.equals("password")) {
+      input = new Password(varName);
     } else {
       throw new RuntimeException("Could not recognize dynamic form with type: " + type);
     }
     input.setArgument(arg);
-    input.setDisplayName(displayName);
+    if (!StringUtils.isBlank(displayName)) {
+      // only set displayName when it is not empty (user explicitly specify it)
+      // e.g. ${name(display_name)=value)
+      input.setDisplayName(displayName);
+    }
     input.setHidden(hidden);
     return input;
   }
@@ -338,18 +354,26 @@ public class Input<T> implements Serializable {
         Collection<Object> checked = value instanceof Collection ? (Collection<Object>) value
             : Arrays.asList((Object[]) value);
         List<Object> validChecked = new LinkedList<>();
-        for (Object o : checked) {  // filter out obsolete checked values
-          for (ParamOption option : optionInput.getOptions()) {
-            if (option.getValue().equals(o)) {
-              validChecked.add(o);
-              break;
+        for (Object o : checked) {
+          // filter out obsolete checked values
+          if (optionInput.getOptions() != null) {
+            for (ParamOption option : optionInput.getOptions()) {
+              if (option.getValue().equals(o)) {
+                validChecked.add(o);
+                break;
+              }
             }
           }
         }
-        params.put(input.name, validChecked);
-        expanded = StringUtils.join(validChecked, delimiter);
-      } else {  // single-selection
-        expanded = value.toString();
+        if (validChecked.isEmpty()) {
+          expanded = StringUtils.join(checked, delimiter);
+        } else {
+          params.put(input.name, validChecked);
+          expanded = StringUtils.join(validChecked, delimiter);
+        }
+      } else {
+        // single-selection
+        expanded = StringUtils.defaultString((String) value, "");
       }
       replaced = match.replaceFirst(expanded);
       match = pattern.matcher(replaced);
