@@ -30,6 +30,7 @@ import org.slf4j.LoggerFactory;
 
 import java.io.File;
 import java.io.IOException;
+import java.nio.charset.StandardCharsets;
 import java.util.List;
 import java.util.Map;
 
@@ -58,9 +59,9 @@ import org.apache.zeppelin.server.JsonResponse;
 public class HeliumRestApi {
   Logger logger = LoggerFactory.getLogger(HeliumRestApi.class);
 
-  private Helium helium;
-  private Notebook notebook;
-  private Gson gson = new Gson();
+  private final Helium helium;
+  private final Notebook notebook;
+  private final Gson gson = new Gson();
 
   @Inject
   public HeliumRestApi(Helium helium, Notebook notebook) {
@@ -78,7 +79,7 @@ public class HeliumRestApi {
       return new JsonResponse<>(Response.Status.OK, "", helium.getAllPackageInfo()).build();
     } catch (RuntimeException e) {
       logger.error(e.getMessage(), e);
-      return new JsonResponse(Response.Status.INTERNAL_SERVER_ERROR, e.getMessage()).build();
+      return new JsonResponse<>(Response.Status.INTERNAL_SERVER_ERROR, e.getMessage()).build();
     }
   }
 
@@ -92,7 +93,7 @@ public class HeliumRestApi {
       return new JsonResponse<>(Response.Status.OK, "", helium.getAllEnabledPackages()).build();
     } catch (RuntimeException e) {
       logger.error(e.getMessage(), e);
-      return new JsonResponse(Response.Status.INTERNAL_SERVER_ERROR, e.getMessage()).build();
+      return new JsonResponse<>(Response.Status.INTERNAL_SERVER_ERROR, e.getMessage()).build();
     }
   }
 
@@ -103,7 +104,7 @@ public class HeliumRestApi {
   @Path("package/{packageName}")
   public Response getSinglePackageInfo(@PathParam("packageName") String packageName) {
     if (StringUtils.isEmpty(packageName)) {
-      return new JsonResponse(Response.Status.BAD_REQUEST,
+      return new JsonResponse<>(Response.Status.BAD_REQUEST,
               "Can't get package info for empty name").build();
     }
 
@@ -112,7 +113,7 @@ public class HeliumRestApi {
           Response.Status.OK, "", helium.getSinglePackageInfo(packageName)).build();
     } catch (RuntimeException e) {
       logger.error(e.getMessage(), e);
-      return new JsonResponse(Response.Status.INTERNAL_SERVER_ERROR, e.getMessage()).build();
+      return new JsonResponse<>(Response.Status.INTERNAL_SERVER_ERROR, e.getMessage()).build();
     }
   }
 
@@ -120,27 +121,27 @@ public class HeliumRestApi {
   @Path("suggest/{noteId}/{paragraphId}")
   public Response suggest(@PathParam("noteId") String noteId,
           @PathParam("paragraphId") String paragraphId) {
-    Note note = null;
+    Note note;
     try {
       note = notebook.getNote(noteId);
     } catch (IOException e) {
-      return new JsonResponse(Response.Status.NOT_FOUND,
+      return new JsonResponse<>(Response.Status.NOT_FOUND,
               "Fail to get note: " + noteId + "\n" + ExceptionUtils.getStackTrace(e)).build();
     }
     if (note == null) {
-      return new JsonResponse(Response.Status.NOT_FOUND, "Note " + noteId + " not found").build();
+      return new JsonResponse<>(Response.Status.NOT_FOUND, "Note " + noteId + " not found").build();
     }
 
     Paragraph paragraph = note.getParagraph(paragraphId);
     if (paragraph == null) {
-      return new JsonResponse(Response.Status.NOT_FOUND, "Paragraph " + paragraphId + " not found")
+      return new JsonResponse<>(Response.Status.NOT_FOUND, "Paragraph " + paragraphId + " not found")
           .build();
     }
     try {
       return new JsonResponse<>(Response.Status.OK, "", helium.suggestApp(paragraph)).build();
     } catch (RuntimeException e) {
       logger.error(e.getMessage(), e);
-      return new JsonResponse(Response.Status.INTERNAL_SERVER_ERROR, e.getMessage()).build();
+      return new JsonResponse<>(Response.Status.INTERNAL_SERVER_ERROR, e.getMessage()).build();
     }
 
   }
@@ -149,20 +150,20 @@ public class HeliumRestApi {
   @Path("load/{noteId}/{paragraphId}")
   public Response load(@PathParam("noteId") String noteId,
           @PathParam("paragraphId") String paragraphId, String heliumPackage) {
-    Note note = null;
+    Note note;
     try {
       note = notebook.getNote(noteId);
     } catch (IOException e) {
-      return new JsonResponse(Response.Status.NOT_FOUND,
+      return new JsonResponse<>(Response.Status.NOT_FOUND,
               "Fail to get note: " + noteId + "\n" + ExceptionUtils.getStackTrace(e)).build();
     }
     if (note == null) {
-      return new JsonResponse(Response.Status.NOT_FOUND, "Note " + noteId + " not found").build();
+      return new JsonResponse<>(Response.Status.NOT_FOUND, "Note " + noteId + " not found").build();
     }
 
     Paragraph paragraph = note.getParagraph(paragraphId);
     if (paragraph == null) {
-      return new JsonResponse(Response.Status.NOT_FOUND, "Paragraph " + paragraphId + " not found")
+      return new JsonResponse<>(Response.Status.NOT_FOUND, "Paragraph " + paragraphId + " not found")
           .build();
     }
     HeliumPackage pkg = HeliumPackage.fromJson(heliumPackage);
@@ -171,7 +172,7 @@ public class HeliumRestApi {
               helium.getApplicationFactory().loadAndRun(pkg, paragraph)).build();
     } catch (RuntimeException e) {
       logger.error(e.getMessage(), e);
-      return new JsonResponse(Response.Status.INTERNAL_SERVER_ERROR, e.getMessage()).build();
+      return new JsonResponse<>(Response.Status.INTERNAL_SERVER_ERROR, e.getMessage()).build();
     }
   }
 
@@ -181,7 +182,7 @@ public class HeliumRestApi {
   public Response bundleLoad(@QueryParam("refresh") String refresh,
           @PathParam("packageName") String packageName) {
     if (StringUtils.isEmpty(packageName)) {
-      return new JsonResponse(
+      return new JsonResponse<>(
           Response.Status.BAD_REQUEST,
           "Can't get bundle due to empty package name").build();
     }
@@ -208,7 +209,7 @@ public class HeliumRestApi {
       if (bundle == null) {
         return Response.ok().build();
       } else {
-        String stringified = FileUtils.readFileToString(bundle);
+        String stringified = FileUtils.readFileToString(bundle, StandardCharsets.UTF_8);
         return Response.ok(stringified).build();
       }
     } catch (Exception e) {
@@ -225,13 +226,13 @@ public class HeliumRestApi {
   public Response enablePackage(@PathParam("packageName") String packageName, String artifact) {
     try {
       if (helium.enable(packageName, artifact)) {
-        return new JsonResponse(Response.Status.OK).build();
+        return new JsonResponse<>(Response.Status.OK).build();
       } else {
-        return new JsonResponse(Response.Status.NOT_FOUND).build();
+        return new JsonResponse<>(Response.Status.NOT_FOUND).build();
       }
     } catch (IOException e) {
       logger.error(e.getMessage(), e);
-      return new JsonResponse(Response.Status.INTERNAL_SERVER_ERROR, e.getMessage()).build();
+      return new JsonResponse<>(Response.Status.INTERNAL_SERVER_ERROR, e.getMessage()).build();
     }
   }
 
@@ -240,13 +241,13 @@ public class HeliumRestApi {
   public Response disablePackage(@PathParam("packageName") String packageName) {
     try {
       if (helium.disable(packageName)) {
-        return new JsonResponse(Response.Status.OK).build();
+        return new JsonResponse<>(Response.Status.OK).build();
       } else {
-        return new JsonResponse(Response.Status.NOT_FOUND).build();
+        return new JsonResponse<>(Response.Status.NOT_FOUND).build();
       }
     } catch (IOException e) {
       logger.error(e.getMessage(), e);
-      return new JsonResponse(Response.Status.INTERNAL_SERVER_ERROR, e.getMessage()).build();
+      return new JsonResponse<>(Response.Status.INTERNAL_SERVER_ERROR, e.getMessage()).build();
     }
   }
 
@@ -254,7 +255,7 @@ public class HeliumRestApi {
   @Path("spell/config/{packageName}")
   public Response getSpellConfigUsingMagic(@PathParam("packageName") String packageName) {
     if (StringUtils.isEmpty(packageName)) {
-      return new JsonResponse(Response.Status.BAD_REQUEST, "packageName is empty").build();
+      return new JsonResponse<>(Response.Status.BAD_REQUEST, "packageName is empty").build();
     }
 
     try {
@@ -262,14 +263,14 @@ public class HeliumRestApi {
           helium.getSpellConfig(packageName);
 
       if (config == null) {
-        return new JsonResponse(Response.Status.BAD_REQUEST,
+        return new JsonResponse<>(Response.Status.BAD_REQUEST,
             "Failed to find enabled package for " + packageName).build();
       }
 
       return new JsonResponse<>(Response.Status.OK, config).build();
     } catch (RuntimeException e) {
       logger.error(e.getMessage(), e);
-      return new JsonResponse(Response.Status.INTERNAL_SERVER_ERROR, e.getMessage()).build();
+      return new JsonResponse<>(Response.Status.INTERNAL_SERVER_ERROR, e.getMessage()).build();
     }
   }
 
@@ -281,7 +282,7 @@ public class HeliumRestApi {
       return new JsonResponse<>(Response.Status.OK, config).build();
     } catch (RuntimeException e) {
       logger.error(e.getMessage(), e);
-      return new JsonResponse(Response.Status.INTERNAL_SERVER_ERROR, e.getMessage()).build();
+      return new JsonResponse<>(Response.Status.INTERNAL_SERVER_ERROR, e.getMessage()).build();
     }
   }
 
@@ -290,7 +291,7 @@ public class HeliumRestApi {
   public Response getPackageConfig(@PathParam("packageName") String packageName,
           @PathParam("artifact") String artifact) {
     if (StringUtils.isEmpty(packageName) || StringUtils.isEmpty(artifact)) {
-      return new JsonResponse(Response.Status.BAD_REQUEST,
+      return new JsonResponse<>(Response.Status.BAD_REQUEST,
           "package name or artifact is empty"
       ).build();
     }
@@ -300,14 +301,14 @@ public class HeliumRestApi {
           helium.getPackageConfig(packageName, artifact);
 
       if (config == null) {
-        return new JsonResponse(Response.Status.BAD_REQUEST,
+        return new JsonResponse<>(Response.Status.BAD_REQUEST,
             "Failed to find package for " + artifact).build();
       }
 
       return new JsonResponse<>(Response.Status.OK, config).build();
     } catch (RuntimeException e) {
       logger.error(e.getMessage(), e);
-      return new JsonResponse(Response.Status.INTERNAL_SERVER_ERROR, e.getMessage()).build();
+      return new JsonResponse<>(Response.Status.INTERNAL_SERVER_ERROR, e.getMessage()).build();
     }
   }
 
@@ -316,7 +317,7 @@ public class HeliumRestApi {
   public Response updatePackageConfig(@PathParam("packageName") String packageName,
           @PathParam("artifact") String artifact, String rawConfig) {
     if (StringUtils.isEmpty(packageName) || StringUtils.isEmpty(artifact)) {
-      return new JsonResponse(Response.Status.BAD_REQUEST,
+      return new JsonResponse<>(Response.Status.BAD_REQUEST,
           "package name or artifact is empty"
       ).build();
     }
@@ -328,10 +329,10 @@ public class HeliumRestApi {
       return new JsonResponse<>(Response.Status.OK, packageConfig).build();
     } catch (JsonParseException e) {
       logger.error(e.getMessage(), e);
-      return new JsonResponse(Response.Status.BAD_REQUEST,
+      return new JsonResponse<>(Response.Status.BAD_REQUEST,
           e.getMessage()).build();
     } catch (IOException | RuntimeException e) {
-      return new JsonResponse(Response.Status.INTERNAL_SERVER_ERROR,
+      return new JsonResponse<>(Response.Status.INTERNAL_SERVER_ERROR,
           e.getMessage()).build();
     }
   }
@@ -344,7 +345,7 @@ public class HeliumRestApi {
       return new JsonResponse<>(Response.Status.OK, order).build();
     } catch (RuntimeException e) {
       logger.error(e.getMessage(), e);
-      return new JsonResponse(Response.Status.INTERNAL_SERVER_ERROR, e.getMessage()).build();
+      return new JsonResponse<>(Response.Status.INTERNAL_SERVER_ERROR, e.getMessage()).build();
     }
   }
 
@@ -355,10 +356,10 @@ public class HeliumRestApi {
         orderedPackageNameList, new TypeToken<List<String>>(){}.getType());
     try {
       helium.setVisualizationPackageOrder(orderedList);
-      return new JsonResponse(Response.Status.OK).build();
+      return new JsonResponse<>(Response.Status.OK).build();
     } catch (IOException e) {
       logger.error(e.getMessage(), e);
-      return new JsonResponse(Response.Status.INTERNAL_SERVER_ERROR, e.getMessage()).build();
+      return new JsonResponse<>(Response.Status.INTERNAL_SERVER_ERROR, e.getMessage()).build();
     }
   }
 }
