@@ -23,7 +23,6 @@ import javax.inject.Singleton;
 import org.apache.commons.lang3.exception.ExceptionUtils;
 import org.apache.zeppelin.annotation.ZeppelinApi;
 import org.apache.zeppelin.dep.Repository;
-import org.apache.zeppelin.interpreter.ExecutionContextBuilder;
 import org.apache.zeppelin.interpreter.InterpreterException;
 import org.apache.zeppelin.interpreter.InterpreterPropertyType;
 import org.apache.zeppelin.interpreter.InterpreterSetting;
@@ -187,7 +186,7 @@ public class InterpreterRestApi {
   public Response removeSetting(@PathParam("settingId") String settingId) throws IOException {
     LOGGER.info("Remove interpreterSetting {}", settingId);
     interpreterSettingManager.remove(settingId);
-    return new JsonResponse(Status.OK).build();
+    return new JsonResponse<>(Status.OK).build();
   }
 
   /**
@@ -213,11 +212,7 @@ public class InterpreterRestApi {
         if (authorizationService.hasRunPermission(entities, noteId) ||
                 authorizationService.hasWritePermission(entities, noteId) ||
                 authorizationService.isOwner(entities, noteId)) {
-          interpreterSettingManager.restart(settingId,
-                  new ExecutionContextBuilder()
-                          .setUser(authenticationService.getPrincipal())
-                          .setNoteId(noteId)
-                          .createExecutionContext());
+          interpreterSettingManager.restart(settingId, authenticationService.getPrincipal(), noteId);
         } else {
           return new JsonResponse<>(Status.FORBIDDEN, "No privilege to restart interpreter")
                   .build();
@@ -274,7 +269,7 @@ public class InterpreterRestApi {
       return new JsonResponse<>(Status.INTERNAL_SERVER_ERROR, e.getMessage(),
           ExceptionUtils.getStackTrace(e)).build();
     }
-    return new JsonResponse(Status.OK).build();
+    return new JsonResponse<>(Status.OK).build();
   }
 
   /**
@@ -294,7 +289,7 @@ public class InterpreterRestApi {
       return new JsonResponse<>(Status.INTERNAL_SERVER_ERROR, e.getMessage(),
           ExceptionUtils.getStackTrace(e)).build();
     }
-    return new JsonResponse(Status.OK).build();
+    return new JsonResponse<>(Status.OK).build();
   }
 
   /**
