@@ -14,22 +14,21 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-
 package org.apache.zeppelin.rest.message;
 
 import com.google.gson.Gson;
+
 import org.apache.zeppelin.common.JsonSerializable;
 
 /**
- * RestartInterpreter rest api request message
+ * RestartInterpreter rest api request message.
  */
 public class RestartInterpreterRequest implements JsonSerializable {
-  private static final Gson gson = new Gson();
+  private static final Gson GSON = new Gson();
 
-  String noteId;
+  private String noteId;
 
   public RestartInterpreterRequest() {
-
   }
 
   public String getNoteId() {
@@ -37,10 +36,10 @@ public class RestartInterpreterRequest implements JsonSerializable {
   }
 
   public String toJson() {
-    return gson.toJson(this);
+    return GSON.toJson(this);
   }
 
   public static RestartInterpreterRequest fromJson(String json) {
-    return gson.fromJson(json, RestartInterpreterRequest.class);
+    return GSON.fromJson(json, RestartInterpreterRequest.class);
   }
 }
