@@ -21,11 +21,14 @@ package org.apache.zeppelin.service;
 
 import static org.apache.zeppelin.conf.ZeppelinConfiguration.ConfVars.ZEPPELIN_NOTEBOOK_HOMESCREEN;
 import static org.apache.zeppelin.interpreter.InterpreterResult.Code.ERROR;
+import static org.apache.zeppelin.scheduler.Job.Status.ABORT;
 
 import com.google.common.base.Strings;
 import java.io.IOException;
 import java.text.ParseException;
 import java.text.SimpleDateFormat;
+import java.time.Instant;
+import java.time.format.DateTimeFormatter;
 import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.LinkedList;
@@ -47,6 +50,8 @@ import org.apache.zeppelin.notebook.NoteManager;
 import org.apache.zeppelin.notebook.Notebook;
 import org.apache.zeppelin.notebook.Paragraph;
 import org.apache.zeppelin.notebook.AuthorizationService;
+import org.apache.zeppelin.notebook.exception.CorruptedNoteException;
+import org.apache.zeppelin.notebook.exception.NotePathAlreadyExistsException;
 import org.apache.zeppelin.notebook.repo.NotebookRepoWithVersionControl;
 import org.apache.zeppelin.notebook.scheduler.SchedulerService;
 import org.apache.zeppelin.common.Message;
@@ -57,9 +62,6 @@ import org.apache.zeppelin.rest.exception.ParagraphNotFoundException;
 import org.apache.zeppelin.scheduler.Job;
 import org.apache.zeppelin.user.AuthenticationInfo;
 import org.bitbucket.cowwoc.diffmatchpatch.DiffMatchPatch;
-import org.joda.time.DateTime;
-import org.joda.time.format.DateTimeFormat;
-import org.joda.time.format.DateTimeFormatter;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -77,12 +79,12 @@ public class NotebookService {
 
   private static final Logger LOGGER = LoggerFactory.getLogger(NotebookService.class);
   private static final DateTimeFormatter TRASH_CONFLICT_TIMESTAMP_FORMATTER =
-      DateTimeFormat.forPattern("yyyy-MM-dd HH:mm:ss");
+          DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
 
-  private ZeppelinConfiguration zConf;
-  private Notebook notebook;
-  private AuthorizationService authorizationService;
-  private SchedulerService schedulerService;
+  private final ZeppelinConfiguration zConf;
+  private final Notebook notebook;
+  private final AuthorizationService authorizationService;
+  private final SchedulerService schedulerService;
 
   @Inject
   public NotebookService(
@@ -116,7 +118,14 @@ public class NotebookService {
   public Note getNote(String noteId,
                       ServiceContext context,
                       ServiceCallback<Note> callback) throws IOException {
-    Note note = notebook.getNote(noteId);
+    return getNote(noteId, false, context, callback);
+  }
+
+  public Note getNote(String noteId,
+                      boolean reload,
+                      ServiceContext context,
+                      ServiceCallback<Note> callback) throws IOException {
+    Note note = notebook.getNote(noteId, reload);
     if (note == null) {
       callback.onFailure(new NoteNotFoundException(noteId), context);
       return null;
@@ -201,11 +210,20 @@ public class NotebookService {
   public void removeNote(String noteId,
                          ServiceContext context,
                          ServiceCallback<String> callback) throws IOException {
-    Note note = notebook.getNote(noteId);
+    if (!checkPermission(noteId, Permission.OWNER, Message.OP.DEL_NOTE, context, callback)) {
+      return;
+    }
+
+    Note note = null;
+    try {
+      note = notebook.getNote(noteId);
+    } catch (CorruptedNoteException e) {
+      notebook.removeCorruptedNote(noteId, context.getAutheInfo());
+      callback.onSuccess("Delete note successfully", context);
+      return;
+    }
+
     if (note != null) {
-      if (!checkPermission(note.getId(), Permission.OWNER, Message.OP.DEL_NOTE, context, callback)) {
-        return;
-      }
       notebook.removeNote(note, context.getAutheInfo());
       callback.onSuccess("Delete note successfully", context);
     } else {
@@ -248,8 +266,12 @@ public class NotebookService {
           newNotePath = "/" + newNotePath;
         }
       }
-      notebook.moveNote(noteId, newNotePath, context.getAutheInfo());
-      callback.onSuccess(note, context);
+      try {
+        notebook.moveNote(noteId, newNotePath, context.getAutheInfo());
+        callback.onSuccess(note, context);
+      } catch (NotePathAlreadyExistsException e) {
+        callback.onFailure(e, context);
+      }
     } else {
       callback.onFailure(new NoteNotFoundException(noteId), context);
     }
@@ -321,7 +343,7 @@ public class NotebookService {
                               ServiceContext context,
                               ServiceCallback<Paragraph> callback) throws IOException {
 
-    LOGGER.info("Start to run paragraph: " + paragraphId + " of note: " + noteId);
+    LOGGER.info("Start to run paragraph: {} of note: {}", paragraphId, noteId);
     if (!checkPermission(noteId, Permission.RUNNER, Message.OP.RUN_PARAGRAPH, context, callback)) {
       return false;
     }
@@ -412,7 +434,7 @@ public class NotebookService {
         for (Map<String, Object> raw : paragraphs) {
           String paragraphId = (String) raw.get("id");
           if (paragraphId == null) {
-            LOGGER.warn("No id found in paragraph json: " + raw);
+            LOGGER.warn("No id found in paragraph json: {}", raw);
             continue;
           }
           try {
@@ -432,6 +454,9 @@ public class NotebookService {
             if (result != null && result.code() == ERROR) {
               return false;
             }
+            if (p.getStatus() == ABORT || p.isAborted()) {
+              return false;
+            }
           } catch (Exception e) {
             throw new IOException("Fail to run paragraph json: " + raw, e);
           }
@@ -445,7 +470,7 @@ public class NotebookService {
         note.runAll(context.getAutheInfo(), true, false, new HashMap<>());
         return true;
       } catch (Exception e) {
-        LOGGER.warn("Fail to run note: " + note.getName(), e);
+        LOGGER.warn("Fail to run note: {}", note.getName(), e);
         return false;
       }
     }
@@ -588,7 +613,7 @@ public class NotebookService {
 
 
   public void restoreAll(ServiceContext context,
-                         ServiceCallback callback) throws IOException {
+                         ServiceCallback<?> callback) throws IOException {
 
     try {
       notebook.restoreAll(context.getAutheInfo());
@@ -651,9 +676,7 @@ public class NotebookService {
       throw new IOException("No such note");
     }
     synchronized (this) {
-      if (note.getParagraphCount() < maxParagraph) {
-        return note.addNewParagraph(context.getAutheInfo());
-      } else {
+      if (note.getParagraphCount() >= maxParagraph) {
         boolean removed = false;
         for (int i = 1; i < note.getParagraphCount(); ++i) {
           if (note.getParagraph(i).getStatus().isCompleted()) {
@@ -665,8 +688,8 @@ public class NotebookService {
         if (!removed) {
           throw new IOException("All the paragraphs are not completed, unable to find available paragraph");
         }
-        return note.addNewParagraph(context.getAutheInfo());
       }
+      return note.addNewParagraph(context.getAutheInfo());
     }
   }
 
@@ -688,7 +711,7 @@ public class NotebookService {
       callback.onFailure(new ParagraphNotFoundException(paragraphId), context);
       return;
     }
-    Paragraph returnedParagraph = null;
+    Paragraph returnedParagraph;
     if (note.isPersonalizedMode()) {
       returnedParagraph = note.clearPersonalizedParagraphOutput(paragraphId,
           context.getAutheInfo().getUser());
@@ -756,8 +779,6 @@ public class NotebookService {
     if (configA.get("cron") != null && configB.get("cron") != null && configA.get("cron")
         .equals(configB.get("cron"))) {
       cronUpdated = true;
-    } else if (configA.get("cron") == null && configB.get("cron") == null) {
-      cronUpdated = false;
     } else if (configA.get("cron") != null || configB.get("cron") != null) {
       cronUpdated = true;
     }
@@ -914,7 +935,7 @@ public class NotebookService {
         callback)) {
       return;
     }
-    Note revisionNote = null;
+    Note revisionNote;
     if (revisionId.equals("Head")) {
       revisionNote = note;
     } else {
@@ -969,7 +990,6 @@ public class NotebookService {
       callback.onSuccess(settings, context);
     } catch (Exception e) {
       callback.onFailure(new IOException("Fail to getEditorSetting", e), context);
-      return;
     }
   }
 
@@ -997,20 +1017,29 @@ public class NotebookService {
   public void moveNoteToTrash(String noteId,
                               ServiceContext context,
                               ServiceCallback<Note> callback) throws IOException {
-    Note note = notebook.getNote(noteId);
+    if (!checkPermission(noteId, Permission.OWNER, Message.OP.MOVE_NOTE_TO_TRASH, context, callback)) {
+      return;
+    }
+
+    String destNotePath = "/" + NoteManager.TRASH_FOLDER + notebook.getNoteManager().getNotesInfo().get(noteId);
+    if (notebook.containsNote(destNotePath)) {
+      destNotePath = destNotePath + " " + TRASH_CONFLICT_TIMESTAMP_FORMATTER.format(Instant.now());
+    }
+
+    Note note = null;
+    try {
+       note = notebook.getNote(noteId);
+    } catch (CorruptedNoteException e) {
+        LOGGER.info("Move corrupted note to trash");
+        notebook.moveNote(noteId, destNotePath, context.getAutheInfo());
+        return;
+    }
+
     if (note == null) {
       callback.onFailure(new NoteNotFoundException(noteId), context);
       return;
     }
 
-    if (!checkPermission(noteId, Permission.OWNER, Message.OP.MOVE_NOTE_TO_TRASH, context,
-        callback)) {
-      return;
-    }
-    String destNotePath = "/" + NoteManager.TRASH_FOLDER + note.getPath();
-    if (notebook.containsNote(destNotePath)) {
-      destNotePath = destNotePath + " " + TRASH_CONFLICT_TIMESTAMP_FORMATTER.print(new DateTime());
-    }
     notebook.moveNote(noteId, destNotePath, context.getAutheInfo());
     callback.onSuccess(note, context);
   }
@@ -1021,12 +1050,12 @@ public class NotebookService {
 
     //TODO(zjffdu) folder permission check
     //TODO(zjffdu) folderPath is relative path, need to fix it in frontend
-    LOGGER.info("Move folder " + folderPath + " to trash");
+    LOGGER.info("Move folder {} to trash", folderPath);
 
     String destFolderPath = "/" + NoteManager.TRASH_FOLDER + "/" + folderPath;
     if (notebook.containsNote(destFolderPath)) {
       destFolderPath = destFolderPath + " " +
-          TRASH_CONFLICT_TIMESTAMP_FORMATTER.print(new DateTime());
+          TRASH_CONFLICT_TIMESTAMP_FORMATTER.format(Instant.now());
     }
 
     notebook.moveFolder("/" + folderPath, destFolderPath, context.getAutheInfo());
