@@ -266,12 +266,12 @@ function NotebookCtrl($scope, $route, $routeParams, $location, $rootScope,
         message: 'Do you still want to export this note?',
         callback: function(result) {
           if (result) {
-            saveAsService.saveAs(jsonContent, $scope.note.name, 'zpln');
+            saveAsService.saveAs(jsonContent, $scope.note.name + '_' + $scope.note.id, 'zpln');
           }
         },
       });
     } else {
-      saveAsService.saveAs(jsonContent, $scope.note.name, 'zpln');
+      saveAsService.saveAs(jsonContent, $scope.note.name + '_' + $scope.note.id, 'zpln');
     }
   };
 
@@ -280,6 +280,11 @@ function NotebookCtrl($scope, $route, $routeParams, $location, $rootScope,
     websocketMsgSrv.convertNote($scope.note.id, $scope.note.name);
   };
 
+  // Export nbformat
+  $scope.reloadNote = function() {
+    websocketMsgSrv.reloadNote($scope.note.id);
+  };
+
   // Clone note
   $scope.cloneNote = function(noteId) {
     BootstrapDialog.confirm({
