@@ -19,6 +19,9 @@ package org.apache.zeppelin.interpreter;
 
 import com.google.gson.Gson;
 import com.google.gson.reflect.TypeToken;
+
+import org.apache.commons.io.FileUtils;
+import org.apache.commons.lang3.StringUtils;
 import org.apache.thrift.TException;
 import org.apache.thrift.server.TThreadPoolServer;
 import org.apache.thrift.transport.TServerSocket;
@@ -35,6 +38,8 @@ import org.apache.zeppelin.interpreter.remote.RemoteInterpreterUtils;
 import org.apache.zeppelin.interpreter.thrift.AppOutputAppendEvent;
 import org.apache.zeppelin.interpreter.thrift.AppOutputUpdateEvent;
 import org.apache.zeppelin.interpreter.thrift.AppStatusUpdateEvent;
+import org.apache.zeppelin.interpreter.thrift.InterpreterRPCException;
+import org.apache.zeppelin.interpreter.thrift.LibraryMetadata;
 import org.apache.zeppelin.interpreter.thrift.ParagraphInfo;
 import org.apache.zeppelin.interpreter.thrift.RegisterInfo;
 import org.apache.zeppelin.interpreter.thrift.OutputAppendEvent;
@@ -43,7 +48,6 @@ import org.apache.zeppelin.interpreter.thrift.OutputUpdateEvent;
 import org.apache.zeppelin.interpreter.thrift.RemoteInterpreterEventService;
 import org.apache.zeppelin.interpreter.thrift.RemoteInterpreterResultMessage;
 import org.apache.zeppelin.interpreter.thrift.RunParagraphsEvent;
-import org.apache.zeppelin.interpreter.thrift.ServiceException;
 import org.apache.zeppelin.interpreter.thrift.WebUrlInfo;
 import org.apache.zeppelin.notebook.Note;
 import org.apache.zeppelin.resource.RemoteResource;
@@ -55,8 +59,11 @@ import org.apache.zeppelin.user.AuthenticationInfo;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
+import java.io.File;
 import java.io.IOException;
 import java.nio.ByteBuffer;
+import java.util.ArrayList;
+import java.util.Collection;
 import java.util.Collections;
 import java.util.LinkedList;
 import java.util.List;
@@ -158,7 +165,7 @@ public class RemoteInterpreterEventServer implements RemoteInterpreterEventServi
   }
 
   @Override
-  public void registerInterpreterProcess(RegisterInfo registerInfo) throws TException {
+  public void registerInterpreterProcess(RegisterInfo registerInfo) throws InterpreterRPCException, TException {
     InterpreterGroup interpreterGroup =
         interpreterSettingManager.getInterpreterGroupById(registerInfo.getInterpreterGroupId());
     if (interpreterGroup == null) {
@@ -179,7 +186,7 @@ public class RemoteInterpreterEventServer implements RemoteInterpreterEventServi
   }
 
   @Override
-  public void unRegisterInterpreterProcess(String intpGroupId) throws TException {
+  public void unRegisterInterpreterProcess(String intpGroupId) throws InterpreterRPCException, TException {
     LOGGER.info("Unregister interpreter process: {}", intpGroupId);
     InterpreterGroup interpreterGroup =
             interpreterSettingManager.getInterpreterGroupById(intpGroupId);
@@ -196,7 +203,7 @@ public class RemoteInterpreterEventServer implements RemoteInterpreterEventServi
   }
 
   @Override
-  public void sendWebUrl(WebUrlInfo weburlInfo) throws TException {
+  public void sendWebUrl(WebUrlInfo weburlInfo) throws InterpreterRPCException, TException {
     InterpreterGroup interpreterGroup =
             interpreterSettingManager.getInterpreterGroupById(weburlInfo.getInterpreterGroupId());
     if (interpreterGroup == null) {
@@ -208,7 +215,7 @@ public class RemoteInterpreterEventServer implements RemoteInterpreterEventServi
   }
 
   @Override
-  public void appendOutput(OutputAppendEvent event) throws TException {
+  public void appendOutput(OutputAppendEvent event) throws InterpreterRPCException, TException {
     if (event.getAppId() == null) {
       runner.appendBuffer(
           event.getNoteId(), event.getParagraphId(), event.getIndex(), event.getData());
@@ -219,7 +226,7 @@ public class RemoteInterpreterEventServer implements RemoteInterpreterEventServi
   }
 
   @Override
-  public void updateOutput(OutputUpdateEvent event) throws TException {
+  public void updateOutput(OutputUpdateEvent event) throws InterpreterRPCException, TException {
     if (event.getAppId() == null) {
       listener.onOutputUpdated(event.getNoteId(), event.getParagraphId(), event.getIndex(),
           InterpreterResult.Type.valueOf(event.getType()), event.getData());
@@ -230,7 +237,7 @@ public class RemoteInterpreterEventServer implements RemoteInterpreterEventServi
   }
 
   @Override
-  public void updateAllOutput(OutputUpdateAllEvent event) throws TException {
+  public void updateAllOutput(OutputUpdateAllEvent event) throws InterpreterRPCException, TException {
     listener.onOutputClear(event.getNoteId(), event.getParagraphId());
     for (int i = 0; i < event.getMsg().size(); i++) {
       RemoteInterpreterResultMessage msg = event.getMsg().get(i);
@@ -240,29 +247,29 @@ public class RemoteInterpreterEventServer implements RemoteInterpreterEventServi
   }
 
   @Override
-  public void appendAppOutput(AppOutputAppendEvent event) throws TException {
+  public void appendAppOutput(AppOutputAppendEvent event) throws InterpreterRPCException, TException {
     appListener.onOutputAppend(event.noteId, event.paragraphId, event.index, event.appId,
         event.data);
   }
 
   @Override
-  public void updateAppOutput(AppOutputUpdateEvent event) throws TException {
+  public void updateAppOutput(AppOutputUpdateEvent event) throws InterpreterRPCException, TException {
     appListener.onOutputUpdated(event.noteId, event.paragraphId, event.index, event.appId,
         InterpreterResult.Type.valueOf(event.type), event.data);
   }
 
   @Override
-  public void updateAppStatus(AppStatusUpdateEvent event) throws TException {
+  public void updateAppStatus(AppStatusUpdateEvent event) throws InterpreterRPCException, TException {
     appListener.onStatusChange(event.noteId, event.paragraphId, event.appId, event.status);
   }
 
   @Override
-  public void checkpointOutput(String noteId, String paragraphId) throws TException {
+  public void checkpointOutput(String noteId, String paragraphId) throws InterpreterRPCException, TException {
     listener.checkpointOutput(noteId, paragraphId);
   }
 
   @Override
-  public void runParagraphs(RunParagraphsEvent event) throws TException {
+  public void runParagraphs(RunParagraphsEvent event) throws InterpreterRPCException, TException {
     try {
       listener.runParagraphs(event.getNoteId(), event.getParagraphIndices(),
           event.getParagraphIds(), event.getCurParagraphId());
@@ -272,12 +279,12 @@ public class RemoteInterpreterEventServer implements RemoteInterpreterEventServi
         LOGGER.info("complete runParagraphs.{}", event);
       }
     } catch (IOException e) {
-      throw new TException(e);
+      throw new InterpreterRPCException(e.toString());
     }
   }
 
   @Override
-  public void addAngularObject(String intpGroupId, String json) throws TException {
+  public void addAngularObject(String intpGroupId, String json) throws InterpreterRPCException, TException {
     LOGGER.debug("Add AngularObject, interpreterGroupId: {}, json: {}", intpGroupId, json);
     AngularObject<?> angularObject = AngularObject.fromJson(json);
     InterpreterGroup interpreterGroup =
@@ -303,12 +310,12 @@ public class RemoteInterpreterEventServer implements RemoteInterpreterEventServi
   }
 
   @Override
-  public void updateAngularObject(String intpGroupId, String json) throws TException {
+  public void updateAngularObject(String intpGroupId, String json) throws InterpreterRPCException, TException {
     AngularObject<?> angularObject = AngularObject.fromJson(json);
     InterpreterGroup interpreterGroup =
         interpreterSettingManager.getInterpreterGroupById(intpGroupId);
     if (interpreterGroup == null) {
-      throw new TException("Invalid InterpreterGroupId: " + intpGroupId);
+      throw new InterpreterRPCException("Invalid InterpreterGroupId: " + intpGroupId);
     }
     AngularObject localAngularObject = interpreterGroup.getAngularObjectRegistry().get(
         angularObject.getName(), angularObject.getNoteId(), angularObject.getParagraphId());
@@ -337,11 +344,11 @@ public class RemoteInterpreterEventServer implements RemoteInterpreterEventServi
   public void removeAngularObject(String intpGroupId,
                                   String noteId,
                                   String paragraphId,
-                                  String name) throws TException {
+                                  String name) throws InterpreterRPCException, TException {
     InterpreterGroup interpreterGroup =
         interpreterSettingManager.getInterpreterGroupById(intpGroupId);
     if (interpreterGroup == null) {
-      throw new TException("Invalid InterpreterGroupId: " + intpGroupId);
+      throw new InterpreterRPCException("Invalid InterpreterGroupId: " + intpGroupId);
     }
     interpreterGroup.getAngularObjectRegistry().remove(name, noteId, paragraphId);
 
@@ -356,11 +363,11 @@ public class RemoteInterpreterEventServer implements RemoteInterpreterEventServi
   }
 
   @Override
-  public void sendParagraphInfo(String intpGroupId, String json) throws TException {
+  public void sendParagraphInfo(String intpGroupId, String json) throws InterpreterRPCException, TException {
     InterpreterGroup interpreterGroup =
         interpreterSettingManager.getInterpreterGroupById(intpGroupId);
     if (interpreterGroup == null) {
-      throw new TException("Invalid InterpreterGroupId: " + intpGroupId);
+      throw new InterpreterRPCException("Invalid InterpreterGroupId: " + intpGroupId);
     }
 
     Map<String, String> paraInfos = GSON.fromJson(json,
@@ -375,7 +382,7 @@ public class RemoteInterpreterEventServer implements RemoteInterpreterEventServi
   }
 
   @Override
-  public List<String> getAllResources(String intpGroupId) throws TException {
+  public List<String> getAllResources(String intpGroupId) throws InterpreterRPCException, TException {
     ResourceSet resourceSet = getAllResourcePoolExcept(intpGroupId);
     List<String> resourceList = new LinkedList<>();
     for (Resource r : resourceSet) {
@@ -385,7 +392,7 @@ public class RemoteInterpreterEventServer implements RemoteInterpreterEventServi
   }
 
   @Override
-  public ByteBuffer getResource(String resourceIdJson) throws TException {
+  public ByteBuffer getResource(String resourceIdJson) throws InterpreterRPCException, TException {
     ResourceId resourceId = ResourceId.fromJson(resourceIdJson);
     Object o = getResource(resourceId);
     ByteBuffer obj;
@@ -395,7 +402,7 @@ public class RemoteInterpreterEventServer implements RemoteInterpreterEventServi
       try {
         obj = Resource.serializeObject(o);
       } catch (IOException e) {
-        throw new TException(e);
+        throw new InterpreterRPCException(e.toString());
       }
     }
     return obj;
@@ -409,7 +416,8 @@ public class RemoteInterpreterEventServer implements RemoteInterpreterEventServi
    * @throws TException
    */
   @Override
-  public ByteBuffer invokeMethod(String intpGroupId, String invokeMethodJson) throws TException {
+  public ByteBuffer invokeMethod(String intpGroupId, String invokeMethodJson)
+          throws InterpreterRPCException, TException {
     InvokeResourceMethodEventMessage invokeMethodMessage =
         InvokeResourceMethodEventMessage.fromJson(invokeMethodJson);
     Object ret = invokeResourceMethod(intpGroupId, invokeMethodMessage);
@@ -428,7 +436,7 @@ public class RemoteInterpreterEventServer implements RemoteInterpreterEventServi
 
   @Override
   public List<ParagraphInfo> getParagraphList(String user, String noteId)
-      throws TException, ServiceException {
+          throws InterpreterRPCException, TException {
     LOGGER.info("get paragraph list from remote interpreter noteId: {}, user = {}",noteId, user);
 
     if (user != null && noteId != null) {
@@ -436,7 +444,7 @@ public class RemoteInterpreterEventServer implements RemoteInterpreterEventServi
       try {
         paragraphInfos = listener.getParagraphList(user, noteId);
       } catch (IOException e) {
-       throw new TException(e);
+       throw new InterpreterRPCException(e.toString());
       }
       return paragraphInfos;
     } else {
@@ -545,7 +553,8 @@ public class RemoteInterpreterEventServer implements RemoteInterpreterEventServi
   @Override
   public void updateParagraphConfig(String noteId,
                                     String paragraphId,
-                                    Map<String, String> config) throws TException {
+                                    Map<String, String> config)
+          throws InterpreterRPCException, TException {
     try {
       Note note = interpreterSettingManager.getNotebook().getNote(noteId);
       note.getParagraph(paragraphId).updateConfig(config);
@@ -554,4 +563,58 @@ public class RemoteInterpreterEventServer implements RemoteInterpreterEventServi
       LOGGER.error("Fail to updateParagraphConfig", e);
     }
   }
+
+  @Override
+  public List<LibraryMetadata> getAllLibraryMetadatas(String interpreter) throws TException {
+    if (StringUtils.isBlank(interpreter)) {
+      LOGGER.warn("Interpreter is blank");
+      return Collections.emptyList();
+    }
+    File interpreterLocalRepo = new File(
+        zConf.getAbsoluteDir(ZeppelinConfiguration.ConfVars.ZEPPELIN_DEP_LOCALREPO)
+            + File.separator
+            + interpreter);
+    if (!interpreterLocalRepo.exists()) {
+      LOGGER.warn("Local interpreter repository {} for interpreter {} doesn't exists", interpreterLocalRepo,
+          interpreter);
+      return Collections.emptyList();
+    }
+    if (!interpreterLocalRepo.isDirectory()) {
+      LOGGER.warn("Local interpreter repository {} is no folder", interpreterLocalRepo);
+      return Collections.emptyList();
+    }
+    Collection<File> files = FileUtils.listFiles(interpreterLocalRepo, new String[] { "jar" }, false);
+    List<LibraryMetadata> metaDatas = new ArrayList<>(files.size());
+    for (File file : files) {
+      try {
+        metaDatas.add(new LibraryMetadata(file.getName(), FileUtils.checksumCRC32(file)));
+      } catch (IOException e) {
+        LOGGER.warn(e.getMessage(), e);
+      }
+    }
+    return metaDatas;
+  }
+
+
+  @Override
+  public ByteBuffer getLibrary(String interpreter, String libraryName) throws TException {
+    if (StringUtils.isAnyBlank(interpreter, libraryName)) {
+      LOGGER.warn("Interpreter \"{}\" or libraryName \"{}\" is blank", interpreter, libraryName);
+      return null;
+    }
+    File library = new File(zConf.getAbsoluteDir(ZeppelinConfiguration.ConfVars.ZEPPELIN_DEP_LOCALREPO)
+        + File.separator + interpreter + File.separator + libraryName);
+    if (!library.exists()) {
+      LOGGER.warn("Library {} doesn't exists", library);
+      return null;
+    }
+
+    try {
+      return ByteBuffer.wrap(FileUtils.readFileToByteArray(library));
+    } catch (IOException e) {
+      LOGGER.error("Unable to read library {}", library, e);
+    }
+    return null;
+  }
+
 }
