@@ -30,7 +30,6 @@ import org.apache.zeppelin.display.AngularObject;
 import org.apache.zeppelin.display.AngularObjectRegistry;
 import org.apache.zeppelin.display.Input;
 import org.apache.zeppelin.interpreter.ExecutionContext;
-import org.apache.zeppelin.interpreter.ExecutionContextBuilder;
 import org.apache.zeppelin.interpreter.Interpreter;
 import org.apache.zeppelin.interpreter.InterpreterFactory;
 import org.apache.zeppelin.interpreter.InterpreterGroup;
@@ -42,6 +41,7 @@ import org.apache.zeppelin.interpreter.ManagedInterpreterGroup;
 import org.apache.zeppelin.interpreter.remote.RemoteAngularObject;
 import org.apache.zeppelin.interpreter.remote.RemoteAngularObjectRegistry;
 import org.apache.zeppelin.interpreter.thrift.InterpreterCompletion;
+import org.apache.zeppelin.notebook.exception.CorruptedNoteException;
 import org.apache.zeppelin.notebook.utility.IdHashes;
 import org.apache.zeppelin.scheduler.ExecutorFactory;
 import org.apache.zeppelin.scheduler.Job.Status;
@@ -151,6 +151,7 @@ public class Note implements JsonSerializable {
   /********************************** transient fields ******************************************/
   private transient boolean loaded = false;
   private transient boolean saved = false;
+  private transient boolean removed = false;
   private transient InterpreterFactory interpreterFactory;
   private transient InterpreterSettingManager interpreterSettingManager;
   private transient ParagraphJobListener paragraphJobListener;
@@ -188,7 +189,7 @@ public class Note implements JsonSerializable {
   }
 
   public String getParentPath() {
-    int pos = path.lastIndexOf("/");
+    int pos = path.lastIndexOf('/');
     if (pos == 0) {
       return "/";
     } else {
@@ -197,7 +198,7 @@ public class Note implements JsonSerializable {
   }
 
   private String getName(String path) {
-    int pos = path.lastIndexOf("/");
+    int pos = path.lastIndexOf('/');
     return path.substring(pos + 1);
   }
 
@@ -329,7 +330,7 @@ public class Note implements JsonSerializable {
         this.path = "/" + name;
       }
     } else {
-      int pos = this.path.lastIndexOf("/");
+      int pos = this.path.lastIndexOf('/');
       this.path = this.path.substring(0, pos + 1) + this.name;
     }
   }
@@ -822,14 +823,10 @@ public class Note implements JsonSerializable {
       if (isolated) {
         LOGGER.info("Releasing interpreters used by this note: {}", id);
         for (InterpreterSetting setting : getUsedInterpreterSettings()) {
-          ExecutionContext executionContext = new ExecutionContextBuilder()
-                  .setUser(authInfo.getUser())
-                  .setNoteId(id)
-                  .setDefaultInterpreterGroup(defaultInterpreterGroup)
-                  .setInIsolatedMode(isolated)
-                  .setStartTime(getStartTime())
-                  .createExecutionContext();
-          setting.closeInterpreters(executionContext);
+          setting.closeInterpreters(getExecutionContext());
+          for (Paragraph p : paragraphs) {
+            p.setInterpreter(null);
+          }
         }
       }
     }
@@ -917,7 +914,7 @@ public class Note implements JsonSerializable {
     }
 
     for (InterpreterSetting setting : settings) {
-      InterpreterGroup intpGroup = setting.getInterpreterGroup(new ExecutionContextBuilder().setUser(user).setNoteId(id).createExecutionContext());
+      InterpreterGroup intpGroup = setting.getInterpreterGroup(getExecutionContext());
       if (intpGroup != null) {
         AngularObjectRegistry registry = intpGroup.getAngularObjectRegistry();
         angularObjects.put(intpGroup.getId(), registry.getAllWithGlobal(id));
@@ -934,10 +931,10 @@ public class Note implements JsonSerializable {
     }
 
     for (InterpreterSetting setting : settings) {
-      if (setting.getInterpreterGroup(new ExecutionContextBuilder().setUser(user).setNoteId(id).createExecutionContext()) == null) {
+      if (setting.getInterpreterGroup(getExecutionContext()) == null) {
         continue;
       }
-      InterpreterGroup intpGroup = setting.getInterpreterGroup(new ExecutionContextBuilder().setUser(user).setNoteId(id).createExecutionContext());
+      InterpreterGroup intpGroup = setting.getInterpreterGroup(getExecutionContext());
       AngularObjectRegistry registry = intpGroup.getAngularObjectRegistry();
 
       if (registry instanceof RemoteAngularObjectRegistry) {
@@ -1083,8 +1080,12 @@ public class Note implements JsonSerializable {
   }
 
   public boolean isIsolatedMode() {
-    return Boolean.parseBoolean(
-            info.getOrDefault("inIsolatedMode", "false").toString());
+    if (info == null) {
+      return false;
+    } else {
+      return Boolean.parseBoolean(
+              info.getOrDefault("inIsolatedMode", "false").toString());
+    }
   }
 
   public void setStartTime(String startTime) {
@@ -1092,7 +1093,11 @@ public class Note implements JsonSerializable {
   }
 
   public String getStartTime() {
-    return info.getOrDefault("startTime", "").toString();
+    if (info == null) {
+      return null;
+    } else {
+      return info.getOrDefault("startTime", "").toString();
+    }
   }
 
   public void clearStartTime() {
@@ -1128,7 +1133,7 @@ public class Note implements JsonSerializable {
    * @return Note
    * @throws IOException if fail to parse note json (note file may be corrupted)
    */
-  public static Note fromJson(String json) throws IOException {
+  public static Note fromJson(String noteId, String json) throws IOException {
     try {
       Note note = GSON.fromJson(json, Note.class);
       convertOldInput(note);
@@ -1137,7 +1142,7 @@ public class Note implements JsonSerializable {
       return note;
     } catch (Exception e) {
       LOGGER.error("Fail to parse note json: {}", e.toString());
-      throw new IOException("Fail to parse note json: " + json, e);
+      throw new CorruptedNoteException(noteId, "Fail to parse note json: " + json, e);
     }
   }
 
@@ -1228,4 +1233,21 @@ public class Note implements JsonSerializable {
   public boolean isSaved() {
     return saved;
   }
+
+  public void setRemoved(boolean removed) {
+    this.removed = removed;
+  }
+
+  public boolean isRemoved() {
+    return removed;
+  }
+
+  public ExecutionContext getExecutionContext() {
+    ExecutionContext executionContext = new ExecutionContext();
+    executionContext.setNoteId(id);
+    executionContext.setDefaultInterpreterGroup(defaultInterpreterGroup);
+    executionContext.setInIsolatedMode(isIsolatedMode());
+    executionContext.setStartTime(getStartTime());
+    return executionContext;
+  }
 }
