@@ -22,92 +22,128 @@ import org.apache.zeppelin.conf.ZeppelinConfiguration;
 import org.apache.zeppelin.conf.ZeppelinConfiguration.ConfVars;
 import org.apache.zeppelin.notebook.Note;
 import org.apache.zeppelin.notebook.NoteInfo;
-import org.apache.zeppelin.notebook.NotebookAuthorization;
+import org.apache.zeppelin.notebook.OldNoteInfo;
 import org.apache.zeppelin.notebook.Paragraph;
+import org.apache.zeppelin.plugin.PluginManager;
 import org.apache.zeppelin.user.AuthenticationInfo;
+import org.apache.zeppelin.util.Util;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
+import javax.inject.Inject;
 import java.io.IOException;
-import java.lang.reflect.Constructor;
-import java.lang.reflect.InvocationTargetException;
-import java.util.*;
+import java.util.ArrayList;
+import java.util.Collections;
+import java.util.Date;
+import java.util.HashMap;
+import java.util.List;
+import java.util.Map;
 
 /**
  * Notebook repository sync with remote storage
  */
 public class NotebookRepoSync implements NotebookRepoWithVersionControl {
-  private static final Logger LOG = LoggerFactory.getLogger(NotebookRepoSync.class);
-  private static final int maxRepoNum = 2;
-  private static final String pushKey = "pushNoteIds";
-  private static final String pullKey = "pullNoteIds";
-  private static final String delDstKey = "delDstNoteIds";
+  private static final Logger LOGGER = LoggerFactory.getLogger(NotebookRepoSync.class);
+  private static final int MAX_REPO_NUM = 2;
+  private static final String PUSH_KEY = "pushNoteIds";
+  private static final String PULL_KEY = "pullNoteIds";
+  private static final String DEL_DST_KEY = "delDstNoteIds";
 
-  private static ZeppelinConfiguration config;
-  private static final String defaultStorage = "org.apache.zeppelin.notebook.repo.GitNotebookRepo";
+  private static final String DEFAULT_STORAGE = "org.apache.zeppelin.notebook.repo.GitNotebookRepo";
 
   private List<NotebookRepo> repos = new ArrayList<>();
-  private final boolean oneWaySync;
+  private boolean oneWaySync;
 
   /**
    * @param conf
    */
   @SuppressWarnings("static-access")
-  public NotebookRepoSync(ZeppelinConfiguration conf) {
-    config = conf;
+  @Inject
+  public NotebookRepoSync(ZeppelinConfiguration conf) throws IOException {
+    init(conf);
+  }
+
+  @Override
+  public void init(ZeppelinConfiguration conf) throws IOException {
     oneWaySync = conf.getBoolean(ConfVars.ZEPPELIN_NOTEBOOK_ONE_WAY_SYNC);
-    String allStorageClassNames = conf.getString(ConfVars.ZEPPELIN_NOTEBOOK_STORAGE).trim();
+    String allStorageClassNames = conf.getNotebookStorageClass().trim();
     if (allStorageClassNames.isEmpty()) {
-      allStorageClassNames = defaultStorage;
-      LOG.warn("Empty ZEPPELIN_NOTEBOOK_STORAGE conf parameter, using default {}", defaultStorage);
+      allStorageClassNames = DEFAULT_STORAGE;
+      LOGGER.warn("Empty ZEPPELIN_NOTEBOOK_STORAGE conf parameter, using default {}",
+              DEFAULT_STORAGE);
     }
     String[] storageClassNames = allStorageClassNames.split(",");
     if (storageClassNames.length > getMaxRepoNum()) {
-      LOG.warn("Unsupported number {} of storage classes in ZEPPELIN_NOTEBOOK_STORAGE : {}\n" +
+      LOGGER.warn("Unsupported number {} of storage classes in ZEPPELIN_NOTEBOOK_STORAGE : {}\n" +
           "first {} will be used", storageClassNames.length, allStorageClassNames, getMaxRepoNum());
     }
 
+    // init the underlying NotebookRepo
     for (int i = 0; i < Math.min(storageClassNames.length, getMaxRepoNum()); i++) {
-      Class<?> notebookStorageClass;
-      try {
-        notebookStorageClass = getClass().forName(storageClassNames[i].trim());
-        Constructor<?> constructor = notebookStorageClass.getConstructor(
-            ZeppelinConfiguration.class);
-        repos.add((NotebookRepo) constructor.newInstance(conf));
-        LOG.info("Instantiate NotebookRepo: " + storageClassNames[i]);
-      } catch (ClassNotFoundException | NoSuchMethodException | SecurityException |
-          InstantiationException | IllegalAccessException | IllegalArgumentException |
-          InvocationTargetException e) {
-        LOG.warn("Failed to initialize {} notebook storage class", storageClassNames[i], e);
-      }
+      NotebookRepo notebookRepo = PluginManager.get().loadNotebookRepo(storageClassNames[i].trim());
+      notebookRepo.init(conf);
+      repos.add(notebookRepo);
     }
+
     // couldn't initialize any storage, use default
     if (getRepoCount() == 0) {
-      LOG.info("No storage could be initialized, using default {} storage", defaultStorage);
-      initializeDefaultStorage(conf);
+      LOGGER.info("No storage could be initialized, using default {} storage", DEFAULT_STORAGE);
+      NotebookRepo defaultNotebookRepo = PluginManager.get().loadNotebookRepo(DEFAULT_STORAGE);
+      defaultNotebookRepo.init(conf);
+      repos.add(defaultNotebookRepo);
     }
+
     // sync for anonymous mode on start
-    if (getRepoCount() > 1 && conf.getBoolean(ConfVars.ZEPPELIN_ANONYMOUS_ALLOWED)) {
+    if (getRepoCount() > 1 && conf.isAnonymousAllowed()) {
       try {
         sync(AuthenticationInfo.ANONYMOUS);
       } catch (IOException e) {
-        LOG.error("Couldn't sync on start ", e);
+        LOGGER.error("Couldn't sync anonymous mode on start ", e);
       }
     }
   }
 
-  @SuppressWarnings("static-access")
-  private void initializeDefaultStorage(ZeppelinConfiguration conf) {
-    Class<?> notebookStorageClass;
-    try {
-      notebookStorageClass = getClass().forName(defaultStorage);
-      Constructor<?> constructor = notebookStorageClass.getConstructor(
-                ZeppelinConfiguration.class);
-      repos.add((NotebookRepo) constructor.newInstance(conf));
-    } catch (ClassNotFoundException | NoSuchMethodException | SecurityException |
-        InstantiationException | IllegalAccessException | IllegalArgumentException |
-        InvocationTargetException e) {
-      LOG.warn("Failed to initialize {} notebook storage class {}", defaultStorage, e);
+  // Zeppelin change its note file name structure in 0.9.0, this is called when upgrading
+  // from 0.9.0 before to 0.9.0 after
+  public void convertNoteFiles(ZeppelinConfiguration conf, boolean deleteOld) throws IOException {
+    // convert old note file (noteId/note.json) to new note file (note_name_note_id.zpln)
+    for (int i = 0; i < repos.size(); ++i) {
+      NotebookRepo newNotebookRepo = repos.get(i);
+      OldNotebookRepo oldNotebookRepo =
+              PluginManager.get().loadOldNotebookRepo(newNotebookRepo.getClass().getCanonicalName());
+      oldNotebookRepo.init(conf);
+      List<OldNoteInfo> oldNotesInfo = oldNotebookRepo.list(AuthenticationInfo.ANONYMOUS);
+      LOGGER.info("Convert old note file to new style, note count: {}", oldNotesInfo.size());
+      LOGGER.info("Delete old note: {}", deleteOld);
+      for (OldNoteInfo oldNoteInfo : oldNotesInfo) {
+        LOGGER.info("Converting note, id: {}", oldNoteInfo.getId());
+        Note note = oldNotebookRepo.get(oldNoteInfo.getId(), AuthenticationInfo.ANONYMOUS);
+        note.setPath(note.getName());
+        note.setVersion(Util.getVersion());
+        newNotebookRepo.save(note, AuthenticationInfo.ANONYMOUS);
+        if (newNotebookRepo instanceof NotebookRepoWithVersionControl) {
+          ((NotebookRepoWithVersionControl) newNotebookRepo).checkpoint(
+                  note.getId(),
+                  note.getPath(),
+                  "Upgrade note '" + note.getName() + "' to " + Util.getVersion(),
+                  AuthenticationInfo.ANONYMOUS);
+        }
+        if (deleteOld) {
+          oldNotebookRepo.remove(note.getId(), AuthenticationInfo.ANONYMOUS);
+          LOGGER.info("Remote old note: {}", note.getId());
+          // TODO(zjffdu) no commit when deleting note, This is an issue of
+          // NotebookRepoWithVersionControl
+          /**
+          if (oldNotebookRepo instanceof NotebookRepoWithVersionControl) {
+            ((NotebookRepoWithVersionControl) oldNotebookRepo).checkpoint(
+                    note.getId(),
+                    note.getName(),
+                    "Delete note '" + note.getName() + "' during note upgrade",
+                    AuthenticationInfo.ANONYMOUS);
+          }
+           **/
+        }
+      }
     }
   }
 
@@ -148,30 +184,30 @@ public class NotebookRepoSync implements NotebookRepoWithVersionControl {
    *  Lists Notebooks from the first repository
    */
   @Override
-  public List<NoteInfo> list(AuthenticationInfo subject) throws IOException {
+  public Map<String, NoteInfo> list(AuthenticationInfo subject) throws IOException {
     return getRepo(0).list(subject);
   }
 
-  /* list from specific repo (for tests) */
+  /* List NoteInfo from specific repo (for tests) */
   List<NoteInfo> list(int repoIndex, AuthenticationInfo subject) throws IOException {
-    return getRepo(repoIndex).list(subject);
+    return new ArrayList<>(getRepo(repoIndex).list(subject).values());
   }
 
   /**
-   *  Returns from Notebook from the first repository
+   * Get Note from the first repository
    */
   @Override
-  public Note get(String noteId, AuthenticationInfo subject) throws IOException {
-    return getRepo(0).get(noteId, subject);
+  public Note get(String noteId, String notePath, AuthenticationInfo subject) throws IOException {
+    return getRepo(0).get(noteId, notePath, subject);
   }
 
-  /* get note from specific repo (for tests) */
-  Note get(int repoIndex, String noteId, AuthenticationInfo subject) throws IOException {
-    return getRepo(repoIndex).get(noteId, subject);
+  /* Get Note from specific repo (for tests) */
+  Note get(int repoIndex, String noteId, String noteName, AuthenticationInfo subject) throws IOException {
+    return getRepo(repoIndex).get(noteId, noteName, subject);
   }
 
   /**
-   *  Saves to all repositories
+   *  Saves note to all repositories
    */
   @Override
   public void save(Note note, AuthenticationInfo subject) throws IOException {
@@ -181,7 +217,7 @@ public class NotebookRepoSync implements NotebookRepoWithVersionControl {
         getRepo(1).save(note, subject);
       }
       catch (IOException e) {
-        LOG.info(e.getMessage() + ": Failed to write to secondary storage");
+        LOGGER.info("{}: Failed to write to secondary storage", e.getMessage());
       }
     }
   }
@@ -192,15 +228,44 @@ public class NotebookRepoSync implements NotebookRepoWithVersionControl {
   }
 
   @Override
-  public void remove(String noteId, AuthenticationInfo subject) throws IOException {
+  public void move(String noteId, String notePath, String newNotePath,
+                   AuthenticationInfo subject) throws IOException {
+    getRepo(0).move(noteId, notePath, newNotePath, subject);
+    if (getRepoCount() > 1) {
+      try {
+        getRepo(1).move(noteId, notePath, newNotePath, subject);
+      }
+      catch (IOException e) {
+        LOGGER.info("{}: Failed to write to secondary storage", e.getMessage());
+      }
+    }
+  }
+
+  @Override
+  public void move(String folderPath, String newFolderPath,
+                   AuthenticationInfo subject) throws IOException {
     for (NotebookRepo repo : repos) {
-      repo.remove(noteId, subject);
+      repo.move(folderPath, newFolderPath, subject);
+    }
+  }
+
+  @Override
+  public void remove(String noteId, String notePath, AuthenticationInfo subject) throws IOException {
+    for (NotebookRepo repo : repos) {
+      repo.remove(noteId, notePath, subject);
     }
     /* TODO(khalid): handle case when removing from secondary storage fails */
   }
 
-  void remove(int repoIndex, String noteId, AuthenticationInfo subject) throws IOException {
-    getRepo(repoIndex).remove(noteId, subject);
+  @Override
+  public void remove(String folderPath, AuthenticationInfo subject) throws IOException {
+    for (NotebookRepo repo : repos) {
+      repo.remove(folderPath, subject);
+    }
+  }
+
+  void remove(int repoIndex, String noteId, String noteName, AuthenticationInfo subject) throws IOException {
+    getRepo(repoIndex).remove(noteId, noteName, subject);
   }
 
   /**
@@ -209,103 +274,70 @@ public class NotebookRepoSync implements NotebookRepoWithVersionControl {
    * @throws IOException
    */
   void sync(int sourceRepoIndex, int destRepoIndex, AuthenticationInfo subject) throws IOException {
-    LOG.info("Sync started");
-    NotebookAuthorization auth = NotebookAuthorization.getInstance();
+    LOGGER.info("Sync started");
     NotebookRepo srcRepo = getRepo(sourceRepoIndex);
     NotebookRepo dstRepo = getRepo(destRepoIndex);
-    List <NoteInfo> allSrcNotes = srcRepo.list(subject);
-    List <NoteInfo> srcNotes = auth.filterByUser(allSrcNotes, subject);
-    List <NoteInfo> dstNotes = dstRepo.list(subject);
+    List<NoteInfo> srcNotes = new ArrayList<>(srcRepo.list(subject).values());
+    List<NoteInfo> dstNotes = new ArrayList<>(dstRepo.list(subject).values());
 
-    Map<String, List<String>> noteIds = notesCheckDiff(srcNotes, srcRepo, dstNotes, dstRepo,
+    Map<String, List<NoteInfo>> noteIds = notesCheckDiff(srcNotes, srcRepo, dstNotes, dstRepo,
         subject);
-    List<String> pushNoteIds = noteIds.get(pushKey);
-    List<String> pullNoteIds = noteIds.get(pullKey);
-    List<String> delDstNoteIds = noteIds.get(delDstKey);
+    List<NoteInfo> pushNoteIds = noteIds.get(PUSH_KEY);
+    List<NoteInfo> pullNoteIds = noteIds.get(PULL_KEY);
+    List<NoteInfo> delDstNoteIds = noteIds.get(DEL_DST_KEY);
 
     if (!pushNoteIds.isEmpty()) {
-      LOG.info("Notes with the following IDs will be pushed");
-      for (String id : pushNoteIds) {
-        LOG.info("ID : " + id);
+      LOGGER.info("The following notes will be pushed");
+      for (NoteInfo noteInfo : pushNoteIds) {
+        LOGGER.info("Note : {}", noteInfo.getId());
       }
-      pushNotes(subject, pushNoteIds, srcRepo, dstRepo, false);
+      pushNotes(subject, pushNoteIds, srcRepo, dstRepo);
     } else {
-      LOG.info("Nothing to push");
+      LOGGER.info("Nothing to push");
     }
 
     if (!pullNoteIds.isEmpty()) {
-      LOG.info("Notes with the following IDs will be pulled");
-      for (String id : pullNoteIds) {
-        LOG.info("ID : " + id);
+      LOGGER.info("The following notes will be pulled");
+      for (NoteInfo noteInfo : pullNoteIds) {
+        LOGGER.info("Note : {}", noteInfo);
       }
-      pushNotes(subject, pullNoteIds, dstRepo, srcRepo, true);
+      pushNotes(subject, pullNoteIds, dstRepo, srcRepo);
     } else {
-      LOG.info("Nothing to pull");
+      LOGGER.info("Nothing to pull");
     }
 
     if (!delDstNoteIds.isEmpty()) {
-      LOG.info("Notes with the following IDs will be deleted from dest");
-      for (String id : delDstNoteIds) {
-        LOG.info("ID : " + id);
+      LOGGER.info("The following notes will be deleted from dest");
+      for (NoteInfo noteInfo : delDstNoteIds) {
+        LOGGER.info("Note : {}", noteInfo.getId());
       }
       deleteNotes(subject, delDstNoteIds, dstRepo);
     } else {
-      LOG.info("Nothing to delete from dest");
+      LOGGER.info("Nothing to delete from dest");
     }
 
-    LOG.info("Sync ended");
+    LOGGER.info("Sync ended");
   }
 
   public void sync(AuthenticationInfo subject) throws IOException {
     sync(0, 1, subject);
   }
 
-  private void pushNotes(AuthenticationInfo subject, List<String> ids, NotebookRepo localRepo,
-      NotebookRepo remoteRepo, boolean setPermissions) {
-    for (String id : ids) {
+  private void pushNotes(AuthenticationInfo subject, List<NoteInfo> notesInfo, NotebookRepo localRepo,
+      NotebookRepo remoteRepo) {
+    for (NoteInfo noteInfo : notesInfo) {
       try {
-        remoteRepo.save(localRepo.get(id, subject), subject);
-        if (setPermissions && emptyNoteAcl(id)) {
-          makePrivate(id, subject);
-        }
+        remoteRepo.save(localRepo.get(noteInfo.getId(), noteInfo.getPath(), subject), subject);
       } catch (IOException e) {
-        LOG.error("Failed to push note to storage, moving onto next one", e);
+        LOGGER.error("Failed to push note to storage, moving onto next one", e);
       }
     }
   }
 
-  private boolean emptyNoteAcl(String noteId) {
-    NotebookAuthorization notebookAuthorization = NotebookAuthorization.getInstance();
-    return notebookAuthorization.getOwners(noteId).isEmpty()
-        && notebookAuthorization.getReaders(noteId).isEmpty()
-            && notebookAuthorization.getRunners(noteId).isEmpty()
-        && notebookAuthorization.getWriters(noteId).isEmpty();
-  }
-
-  private void makePrivate(String noteId, AuthenticationInfo subject) {
-    if (AuthenticationInfo.isAnonymous(subject)) {
-      LOG.info("User is anonymous, permissions are not set for pulled notes");
-      return;
-    }
-    NotebookAuthorization notebookAuthorization = NotebookAuthorization.getInstance();
-    Set<String> users = notebookAuthorization.getOwners(noteId);
-    users.add(subject.getUser());
-    notebookAuthorization.setOwners(noteId, users);
-    users = notebookAuthorization.getReaders(noteId);
-    users.add(subject.getUser());
-    notebookAuthorization.setReaders(noteId, users);
-    users = notebookAuthorization.getRunners(noteId);
-    users.add(subject.getUser());
-    notebookAuthorization.setRunners(noteId, users);
-    users = notebookAuthorization.getWriters(noteId);
-    users.add(subject.getUser());
-    notebookAuthorization.setWriters(noteId, users);
-  }
-
-  private void deleteNotes(AuthenticationInfo subject, List<String> ids, NotebookRepo repo)
+  private void deleteNotes(AuthenticationInfo subject, List<NoteInfo> noteInfos, NotebookRepo repo)
       throws IOException {
-    for (String id : ids) {
-      repo.remove(id, subject);
+    for (NoteInfo noteInfo : noteInfos) {
+      repo.remove(noteInfo.getId(), noteInfo.getPath(), subject);
     }
   }
 
@@ -314,7 +346,7 @@ public class NotebookRepoSync implements NotebookRepoWithVersionControl {
   }
 
   int getMaxRepoNum() {
-    return maxRepoNum;
+    return MAX_REPO_NUM;
   }
 
   public NotebookRepo getRepo(int repoIndex) throws IOException {
@@ -325,12 +357,12 @@ public class NotebookRepoSync implements NotebookRepoWithVersionControl {
     return repos.get(repoIndex);
   }
 
-  private Map<String, List<String>> notesCheckDiff(List<NoteInfo> sourceNotes,
+  private Map<String, List<NoteInfo>> notesCheckDiff(List<NoteInfo> sourceNotes,
       NotebookRepo sourceRepo, List<NoteInfo> destNotes, NotebookRepo destRepo,
       AuthenticationInfo subject) {
-    List <String> pushIDs = new ArrayList<>();
-    List <String> pullIDs = new ArrayList<>();
-    List <String> delDstIDs = new ArrayList<>();
+    List<NoteInfo> pushIDs = new ArrayList<>();
+    List<NoteInfo> pullIDs = new ArrayList<>();
+    List<NoteInfo> delDstIDs = new ArrayList<>();
 
     NoteInfo dnote;
     Date sdate, ddate;
@@ -339,10 +371,10 @@ public class NotebookRepoSync implements NotebookRepoWithVersionControl {
       if (dnote != null) {
         try {
           /* note exists in source and destination storage systems */
-          sdate = lastModificationDate(sourceRepo.get(snote.getId(), subject));
-          ddate = lastModificationDate(destRepo.get(dnote.getId(), subject));
+          sdate = lastModificationDate(sourceRepo.get(snote.getId(), snote.getPath(), subject));
+          ddate = lastModificationDate(destRepo.get(dnote.getId(), dnote.getPath(), subject));
         } catch (IOException e) {
-          LOG.error("Cannot access previously listed note {} from storage ", dnote.getId(), e);
+          LOGGER.error("Cannot access previously listed note {} from storage ", dnote.getId(), e);
           continue;
         }
 
@@ -350,19 +382,19 @@ public class NotebookRepoSync implements NotebookRepoWithVersionControl {
           if (sdate.after(ddate) || oneWaySync) {
             /* if source contains more up to date note - push
              * if oneWaySync is enabled, always push no matter who's newer */
-            pushIDs.add(snote.getId());
-            LOG.info("Modified note is added to push list : " + sdate);
+            pushIDs.add(snote);
+            LOGGER.info("Modified note is added to push list : {}", sdate);
           } else {
             /* destination contains more up to date note - pull */
-            LOG.info("Modified note is added to pull list : " + ddate);
-            pullIDs.add(snote.getId());
+            LOGGER.info("Modified note is added to pull list : {}", ddate);
+            pullIDs.add(snote);
           }
         }
       } else {
         /* note exists in source storage, and absent in destination
          * view source as up to date - push
          * (another scenario : note was deleted from destination - not considered)*/
-        pushIDs.add(snote.getId());
+        pushIDs.add(snote);
       }
     }
 
@@ -372,20 +404,20 @@ public class NotebookRepoSync implements NotebookRepoWithVersionControl {
         /* note exists in destination storage, and absent in source */
         if (oneWaySync) {
           /* if oneWaySync is enabled, delete the note from destination */
-          LOG.info("Extraneous note is added to delete dest list : " + note.getId());
-          delDstIDs.add(note.getId());
+          LOGGER.info("Extraneous note is added to delete dest list : {}", note.getId());
+          delDstIDs.add(note);
         } else {
           /* if oneWaySync is disabled, pull the note from destination */
-          LOG.info("Missing note is added to pull list : " + note.getId());
-          pullIDs.add(note.getId());
+          LOGGER.info("Missing note is added to pull list : {}", note.getId());
+          pullIDs.add(note);
         }
       }
     }
 
-    Map<String, List<String>> map = new HashMap<>();
-    map.put(pushKey, pushIDs);
-    map.put(pullKey, pullIDs);
-    map.put(delDstKey, delDstIDs);
+    Map<String, List<NoteInfo>> map = new HashMap<>();
+    map.put(PUSH_KEY, pushIDs);
+    map.put(PULL_KEY, pullIDs);
+    map.put(DEL_DST_KEY, delDstIDs);
     return map;
   }
 
@@ -425,35 +457,35 @@ public class NotebookRepoSync implements NotebookRepoWithVersionControl {
 
   @Override
   public void close() {
-    LOG.info("Closing all notebook storages");
+    LOGGER.info("Closing all notebook storages");
     for (NotebookRepo repo: repos) {
       repo.close();
     }
   }
 
-  public Boolean isRevisionSupportedInDefaultRepo() {
+  public boolean isRevisionSupportedInDefaultRepo() {
     return isRevisionSupportedInRepo(0);
   }
 
-  public Boolean isRevisionSupportedInRepo(int repoIndex) {
+  public boolean isRevisionSupportedInRepo(int repoIndex) {
     try {
       if (getRepo(repoIndex) instanceof NotebookRepoWithVersionControl) {
         return true;
       }
     } catch (IOException e) {
-      LOG.error("Error getting default repo", e);
+      LOGGER.error("Error getting default repo", e);
     }
     return false;
   }
 
   //checkpoint to all available storages
   @Override
-  public Revision checkpoint(String noteId, String checkpointMsg, AuthenticationInfo subject)
+  public Revision checkpoint(String noteId, String notePath, String checkpointMsg, AuthenticationInfo subject)
       throws IOException {
     int repoCount = getRepoCount();
     int repoBound = Math.min(repoCount, getMaxRepoNum());
     int errorCount = 0;
-    String errorMessage = "";
+    StringBuilder errorMessage = new StringBuilder("");
     List<Revision> allRepoCheckpoints = new ArrayList<>();
     Revision rev = null;
     for (int i = 0; i < repoBound; i++) {
@@ -461,21 +493,21 @@ public class NotebookRepoSync implements NotebookRepoWithVersionControl {
         if (isRevisionSupportedInRepo(i)) {
           allRepoCheckpoints
               .add(((NotebookRepoWithVersionControl) getRepo(i))
-                  .checkpoint(noteId, checkpointMsg, subject));
+                  .checkpoint(noteId, notePath, checkpointMsg, subject));
         }
       } catch (IOException e) {
-        LOG.warn("Couldn't checkpoint in {} storage with index {} for note {}",
-          getRepo(i).getClass().toString(), i, noteId);
-        errorMessage += "Error on storage class " + getRepo(i).getClass().toString() +
-          " with index " + i + " : " + e.getMessage() + "\n";
+        LOGGER.warn("Couldn't checkpoint in {} storage with index {} for note {}",
+          getRepo(i).getClass(), i, noteId);
+        errorMessage.append("Error on storage class " + getRepo(i).getClass().toString() +
+          " with index " + i + " : " + e.getMessage() + "\n");
         errorCount++;
       }
     }
     // throw exception if failed to commit for all initialized repos
     if (errorCount == repoBound) {
-      throw new IOException(errorMessage);
+      throw new IOException(errorMessage.toString());
     }
-    if (allRepoCheckpoints.size() > 0) {
+    if (!allRepoCheckpoints.isEmpty()) {
       rev = allRepoCheckpoints.get(0);
       // if failed to checkpoint on first storage, then return result on second
       if (allRepoCheckpoints.size() > 1 && rev == null) {
@@ -486,27 +518,30 @@ public class NotebookRepoSync implements NotebookRepoWithVersionControl {
   }
 
   @Override
-  public Note get(String noteId, String revId, AuthenticationInfo subject) {
+  public Note get(String noteId, String notePath, String revId, AuthenticationInfo subject) {
     Note revisionNote = null;
     try {
       if (isRevisionSupportedInDefaultRepo()) {
-        revisionNote = ((NotebookRepoWithVersionControl) getRepo(0)).get(noteId, revId, subject);
+        revisionNote = ((NotebookRepoWithVersionControl) getRepo(0)).get(noteId, notePath,
+            revId, subject);
       }
     } catch (IOException e) {
-      LOG.error("Failed to get revision {} of note {}", revId, noteId, e);
+      LOGGER.error("Failed to get revision {} of note {}", revId, noteId, e);
     }
     return revisionNote;
   }
 
   @Override
-  public List<Revision> revisionHistory(String noteId, AuthenticationInfo subject) {
+  public List<Revision> revisionHistory(String noteId, String notePath,
+                                        AuthenticationInfo subject) {
     List<Revision> revisions = Collections.emptyList();
     try {
       if (isRevisionSupportedInDefaultRepo()) {
-        revisions = ((NotebookRepoWithVersionControl) getRepo(0)).revisionHistory(noteId, subject);
+        revisions = ((NotebookRepoWithVersionControl) getRepo(0))
+            .revisionHistory(noteId, notePath, subject);
       }
     } catch (IOException e) {
-      LOG.error("Failed to list revision history", e);
+      LOGGER.error("Failed to list revision history", e);
     }
     return revisions;
   }
@@ -517,7 +552,7 @@ public class NotebookRepoSync implements NotebookRepoWithVersionControl {
     try {
       repoSettings =  getRepo(0).getSettings(subject);
     } catch (IOException e) {
-      LOG.error("Cannot get notebook repo settings", e);
+      LOGGER.error("Cannot get notebook repo settings", e);
     }
     return repoSettings;
   }
@@ -527,21 +562,22 @@ public class NotebookRepoSync implements NotebookRepoWithVersionControl {
     try {
       getRepo(0).updateSettings(settings, subject);
     } catch (IOException e) {
-      LOG.error("Cannot update notebook repo settings", e);
+      LOGGER.error("Cannot update notebook repo settings", e);
     }
   }
 
   @Override
-  public Note setNoteRevision(String noteId, String revId, AuthenticationInfo subject)
+  public Note setNoteRevision(String noteId, String notePath, String revId, AuthenticationInfo subject)
       throws IOException {
     int repoCount = getRepoCount();
     int repoBound = Math.min(repoCount, getMaxRepoNum());
-    Note currentNote = null, revisionNote = null;
+    Note currentNote = null;
+    Note revisionNote = null;
     for (int i = 0; i < repoBound; i++) {
       try {
         if (isRevisionSupportedInRepo(i)) {
           currentNote = ((NotebookRepoWithVersionControl) getRepo(i))
-              .setNoteRevision(noteId, revId, subject);
+              .setNoteRevision(noteId, notePath, revId, subject);
         }
       } catch (IOException e) {
         // already logged
